/*---------------------------------------------------------------------------------------------------*\
|   PROGRAM NAME:                                                                                     |
|   MODULAR PROGRAM 3: FREQUENCY OF SELECT INCIDENT EVENTS DURING EXPOSURE TO A DRUG/PROCEDURE GROUP  |
|   OF INTEREST                                                                                       |
|                                                                                                     |
|   Date: 01/13/2014                                                                                  |
|   Version: 7.1                                                                                      |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   The purpose of this program is to execute requests consistent with                                |
|   Modular Program 3 specifications                                                                  |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Program inputs:                                                                                   |
|       As specified in Modular Program 3 documentation                                               |
|                                                                                                     |
|   Program outputs:                                                                                  |
|       As specified in Modular Program 3 documentation                                               |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Contact:                                                                                          |
|       Mini-Sentinel Coordinating Center                                                             |
|       info@mini-sentinel.org                                                                        |
|                                                                                                     |
\*---------------------------------------------------------------------------------------------------*/

/*System Options*/
options mprint linesize=150 pagesize=50 compress=yes reuse=no symbolgen ERRORS=0 noquotelenmax;


/***********************/
/*    User Inputs      */
/***********************/

/*
1) Edit DPID and Site ID according to the table below
*/

%LET DPID=MS;
%LET SITEID=OC;
 
   /*-------------------------------------------------------------------*\
   ||                                                                   ||
   ||  DATA PARTNERS                                 DPID      SITEID   ||
   ||  -----------------------------------------------------------------||
   ||   Healthcore (one site)                         HC         OS     ||
   ||   Humana (one site)                             HU         OS     ||
   ||   Aetna (one site)                              AE         OS     ||
   ||   Optum (one site)                              OP         OS     ||
   ||   Vanderbilt University (one site)              VB         OS     ||
   ||   HMORN                                                           ||
   ||      Group Health Cooperative                   HM         GHC    ||
   ||      Fallon Community Health Plan               HM         MPCI   ||   
   ||      Henry Ford Health System                   HM         HFHS   || 
   ||      Lovelace Health System                     HM         LCF    || 
   ||      Marshfield Clinic                          HM         MCRF   ||
   ||      HealthPartners                             HM         HPRF   ||
   ||      Harvard Pilgrim Health Care                HM         HPHC   ||
   ||   Kaiser Permanente                                               ||         
   ||      Kaiser DCC                                 KP         DCC    ||
   ||      Kaiser Permanente Colorado                 KP         CO     ||
   ||      Kaiser Permanente Georgia                  KP         GA     ||
   ||      Kaiser Permanente Hawaii                   KP         HI     ||
   ||      Kaiser Permanente Northern California      KP         NC     ||
   ||      Kaiser Permanente Northwest                KP         NW     ||
   ||      Kaiser Permanente Mid Atlantic             KP         MA     ||
   \*-------------------------------------------------------------------*/

/*
2) Edit this section to reflect your name for each Table/File (or View)
*/
   
%let ENRTABLE=Enrollment;
%let DEMTABLE=Demographic;
%let DISTABLE=Dispensing;
%let DIATABLE=Diagnosis;
%let PROCTABLE=Procedure;
%let ENCTABLE=Encounter;
%let VITTABLE=Vital_sign;
   
/*
3) Edit this section to reflect locations for the libraries/folders for Mini-Sentinel Data 
   and Output folders
*/
/********** FOLDER CONTAINING INPUT DATA FILES AND MSCDM DATA ***************************************/
/* IMPORTANT NOTE: end of path separators are needed;                                               */
/*   Windows-based platforms:    "\", e.g. "C:\user\sas\" and not "C:\user\sas";                    */
/*   Unix-based platforms:      "/", e.g."/home/user/sas/" and not "/home/user/sas";                */
/*                                                                                                  */
/********** FOLDER CONTAINING INPUT DATA FILES AND MSCDM DATA ***************************************/;
/*Data in MSCDM Format*/          libname indata 'C:\Harvard\FLM\';									  	
/*NDC/ICD9 Codes File Location*/  %let infolder=C:\Harvard\MP3\InputFiles\;								  
/*SAS Output Files*/              libname infolder "&infolder.";

 /********** FOLDER CONTAINING SUMMARY FILES TO BE EXPORTED TO Mini Sentinel Operations Center (MSOC)*/;
/*CSV Output Files*/              %let MSOC=C:\Harvard\MP3\MSOC\;       
/*SAS Output Files*/              libname MSOC "&MSOC.";

/*********** FOLDER CONTAINING FINAL DATASETS TO BE KEPT LOCAL AT THE PARTNER SITE (DPLocal)**********/;
/*CSV Output Files*/              %let DPLocal=C:\Harvard\MP3\DPLocal\;       
/*SAS Output Files*/              libname DPLocal "&DPLocal.";

/*****************************************************************************************************/
/**************************** PLEASE DO NOT EDIT CODE BELOW THIS LINE ********************************/
/*****************************************************************************************************/

/*MODIF WAVE3 HERE*/
%MACRO MODULARPROGRAM3(REQUESTID=,RUNID=,ENROLGAP=,COVERAGE=,QUERYFROM=,QUERYTO=,QUERYFILE=,INCQUERYFILE=,
                       QUERYEVENTFILE=,INCEVENTFILE=,CONDFILE=,AGESTRAT=,CCIFILE=,UTILFILE=,                      
                       BMIKIDSFILE=,OUTTABLESFILE=,STOCKPILINGFILE=);                       			  

/*---------------------------------------------------------------------------*/
/*  01 -- Set up variables needed for the run                                */
/*---------------------------------------------------------------------------*/

%LET DPID=%LOWCASE(&DPID.);
%LET SITEID=%LOWCASE(&SITEID.);
%LET REQUESTID=%LOWCASE(&REQUESTID.);
%LET RUNID=%LOWCASE(&RUNID.);
%LET COVERAGE=%UPCASE(&COVERAGE.);
%LET QUERYFILE=%LOWCASE(&QUERYFILE.);
%LET INCQUERYFILE=%LOWCASE(&INCQUERYFILE.);
%LET QUERYEVENTFILE=%LOWCASE(&QUERYEVENTFILE.);
%LET CONDFILE=%LOWCASE(&CONDFILE.);
%LET CCIFILE=%LOWCASE(&CCIFILE.);
%LET UTILFILE=%LOWCASE(&UTILFILE.);
%LET BMIKIDSFILE=%LOWCASE(&BMIKIDSFILE.);
%LET OUTTABLESFILE=%LOWCASE(&OUTTABLESFILE.);
/*MODIF WAVE3 HERE*/
%LET STOCKPILINGFILE=%LOWCASE(&STOCKPILINGFILE.);

proc printto log="&MSOC.&REQUESTID.&RUNID..log" new;
run;

%PUT "MODULARPROGRAM3_v7.1";

%let MPVer=7.1;
%let MPNum=3;

/*Empty working files*/
proc datasets NOLIST NOWARN library=WORK;
   delete _:;
quit;

/*---------------------------------------------------------------------------------------------------*\
|   Program name: Attrition table Tool.                                                               |
|                                                                                                     |
|   Date: 04/02/2013                                                                                  |
|   Version: 1.0                                                                                      |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   For a MP or a task order, a cohort is created by applying a set of inclusion/exclusion criteria   |
|   to a group of members. It is often useful to see how a cohort evolves after successively          |
|   applying these filters. The attrition table tool will enable the requester to summarize this      |
|   information in a single output table.                                                             |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Program inputs:                                                                                   |
|                  -SAS data file with inclusion/exclusion criteria dummy variables                   |
|                                                                                                     |
|   Program outputs:                                                                                  |
|                  -SAS data file (.SAS7BDAT format) containing the attrition table                   |
|                                                                                                     |
|   Macro variable definitions:                                                                       |
|   INFILE         = Source data set with inclusion/exclusion criteria dummy variables                |
|   CRITFLAG       = Variable used to identify inclusions/exclusions                                  |
|   CLASSVARS      = Variable used to identify count unit                                             |
|   WHERE          = Variable to identify extra conditions                                            |
|   EXCLUDEDVAR    = Name of the variable used to flag observations as excluded in the INFILE         |
|   OUTFILE        = Name of the attrition table                                                      |
|   CRITDESC       = Title for an inclusion/exclusion in the OUTFILE                                  |
|   CRITNUM        = Inclusion/exclusion number in the OUTFILE                                        |
|   CRITCNTNAME    = Name of the count variable                                                       |
|   ADDITIONALVARS = Additional variables to add to the OUTFILE                                       |
|                                                                                                     |
\*---------------------------------------------------------------------------------------------------*/


%macro MS_ATTRITIONTABLE(INFILE=,CRITFLAG=,CLASSVARS=,WHERE=,EXCLUDEDVAR=,OUTFILE=,
                         CRITDESC=,CRITNUM=,CRITCNTNAME=,ADDITIONALVARS=);

	%macro commonblock(BLOKIN=,BLOCKWHERE=);

		proc sql noprint;	
	  	  /*Count the number of observations where the criterion applies*/
	  	  select sum(&CRITFLAG.) 
	  	  into :SELECT_COUNTS
	  	  from &BLOKIN.
	  	  where &BLOCKWHERE.;

	  	/*Add an empty row into the outfile*/
	  	proc sql noprint;
	  	  insert into &OUTFILE. (&CRITCNTNAME.) values(0);
	  	quit;
		quit;

		/*Update the outfile with the criterion counts*/
		data &OUTFILE.;
		set &OUTFILE. end=eof;
		if eof then do;
			&ADDITIONALVARS.;
			CritNum=&CRITNUM.;
			CritDesc=&CRITDESC.;
			&CRITCNTNAME.=max(&SELECT_COUNTS.,0);		
		end;
		run;

	%mend commonblock;


	%if %upcase(%str("&EXCLUDEDVAR.")) ne %str("") %then %do;
		%if %upcase(%str("&CLASSVARS.")) ne %str("") %then %do;

		    /*Count the number of dimensions in the class variable*/
			%let DIMCNT = %SYSFUNC(COUNTW(&CLASSVARS.));
			%put &DIMCNT;

			data _dimcnt;
        	format classvars $30.;
			do i = 1 to &DIMCNT.;
        		classvars = compress("'"||compress(scan("&CLASSVARS.",&DIMCNT.))||"'");
				output;
			end;
			drop i;
			run;

			proc sql noprint;
		  	  select classvars
   		  	  into :HKVAR separated by ','
   		  	  from _dimcnt;
			quit;
			%put &HKVAR.;
	
			/*To collapse according to class variable*/
			proc means noprint data = &INFILE. nway;
			var &CRITFLAG.;
			class &CLASSVARS.;
			where &where. and not &EXCLUDEDVAR.;
			output out=_tempattrition(drop=_:) max = ;
			run;

			/*Generate counts*/
			%commonblock(BLOKIN=_tempattrition,BLOCKWHERE=&CRITFLAG.);

			/*Update of exclusion flag in the infile*/
			data _exculdedkey;
			set _tempattrition(where=(&CRITFLAG.) keep = &CLASSVARS. &CRITFLAG.);
			rename &CRITFLAG. = _&EXCLUDEDVAR.;
			run;

			data &INFILE.(drop=_&EXCLUDEDVAR.);
   			if 0 then set _exculdedkey;
   			declare hash ht (hashexp:16, dataset:'_exculdedkey');
   			ht.definekey(&HKVAR.);
   			ht.definedata(ALL: 'YES');
   			ht.definedone();

   			do until(eof1);
      			set &INFILE. end=eof1;
      			if ht.find()=0 then do;
         			&EXCLUDEDVAR. = _&EXCLUDEDVAR.;
      			end;
				output;
   			end;
   			stop;
			run;
	
		%end;
		%else %do;

            /*Generate counts*/
			%commonblock(BLOKIN=&INFILE.,BLOCKWHERE=&where. and &CRITFLAG. and not &EXCLUDEDVAR.);

			/*Update of exclusion flag in the infile*/
			data &infile.;
			set &infile.;			
			if &where. and &CRITFLAG. then &EXCLUDEDVAR.=1;		
			run;

		%end;
	%end;
	%else %do;
		%if %upcase(%str("&CLASSVARS.")) ne %str("") %then %do;

            /*Count the number of dimensions in the class variable*/
			%let DIMCNT = %SYSFUNC(COUNTW(&CLASSVARS.));
			%put &DIMCNT;

			data _dimcnt;
        	format classvars $30.;
			do i = 1 to &DIMCNT.;
        		classvars = compress("'"||compress(scan("&CLASSVARS.",&DIMCNT.))||"'");
				output;
			end;
			drop i;
			run;

			proc sql noprint;
		 	  select classvars
   		 	  into :HKVAR separated by ','
   		 	  from _dimcnt;
			quit;
			%put &HKVAR.;
	
			/*To collapse according to class variable*/
			proc means noprint data = &INFILE. nway;
			var &CRITFLAG.;
			class &CLASSVARS.;
			where &where.;
			output out=_tempattrition(drop=_:) max = ;
			run;

			/*Generate counts*/
			%commonblock(BLOKIN=_tempattrition,BLOCKWHERE=&CRITFLAG.);
	
		%end;
		%else %do;
 
		    /*Generate counts*/
			%commonblock(BLOKIN=&INFILE.,BLOCKWHERE=&where. and &CRITFLAG.);
	
		%end;
	%end;

%mend MS_ATTRITIONTABLE;
 
/*Set default values*/
%MACRO WRAPPER;
   %IF %STR("&AGESTRAT.")=%STR("") %THEN %DO;
      %LET AGESTRAT=00-01 02-04 05-09 10-14 15-18 19-21 22-44 45-64 65-74 75+;
   %END;    
   %IF %STR("&ENROLGAP.")=%STR("") %THEN %DO;
      %LET ENROLGAP=0;
   %END;      
   %IF %STR("&COVERAGE.")=%STR("") OR (%STR("&COVERAGE.") ne %STR("MD") AND %STR("&COVERAGE.") ne %STR("M") AND %STR("&COVERAGE.") ne %STR("D")) %THEN %DO;
      %LET COVERAGE=MD;
   %END;  
%MEND WRAPPER;
%WRAPPER;
   
/*************************/
/* Age Buckets Transform */
/*************************/

/*Macro to count words in a macro VECTOR*/
%MACRO WORDNUMARG(VECTOR,OUTNAME); 
   %GLOBAL &OUTNAME.;
   %LET NUMARG=0; 
   %DO %WHILE(%QSCAN(&VECTOR,&NUMARG+1,%STR( )) ne %STR());
      %LET NUMARG = %EVAL(&NUMARG+1);
   %END;
   %LET &OUTNAME.=&NUMARG.;
%MEND WORDNUMARG;
%WORDNUMARG(&AGESTRAT.,NUMAGECAT);  /*Age Stratification*/
%PUT &NUMAGECAT.;

data _null_;
   format AGETHRESH $1000. AGETYP $1000.;
   do i=1 to &NUMAGECAT.;
      _agetyp=compress(scan("&AGESTRAT.",i*2-1),'DWMQY','klu');

      _agetyp=TRANWRD(UPCASE(_agetyp), 'Y','Years');
      _agetyp=TRANWRD(UPCASE(_agetyp), 'D','Days');
      _agetyp=TRANWRD(UPCASE(_agetyp), 'W','Weeks');
      _agetyp=TRANWRD(UPCASE(_agetyp), 'Q','Quarters');
      _agetyp=TRANWRD(UPCASE(_agetyp), 'M','Months');      

      if _agetyp ne '' then    AGETYP=strip(AGETYP)||" "||strip(_AGETYP);
      else    AGETYP=strip(AGETYP)||' Years';

      AGETHRESH=strip(AGETHRESH)||" "||compress(scan("&AGESTRAT.",i*2-1),'DWMQY','lu');
       if i=1 then MINAGE=AGETHRESH;
      if i=&NUMAGECAT. then do;
         MAXAGE=compress(scan("&AGESTRAT.",i*2),'DWMQY','l')*1;
         if MAXAGE='' then MAXAGE=99999;
      end;
      output;
   end;
   call symput('AGETHRESH',strip(AGETHRESH));
   call symput('AGETYP',strip(AGETYP));
   call symput('MINAGE',strip(MINAGE));
   call symput('MAXAGE',strip(MAXAGE));
run;

%PUT &AGETHRESH.;
%PUT &AGETYP.;
%PUT &NUMAGECAT.;
%PUT &MINAGE;
%PUT &MAXAGE.;

/*Set macro variables for Query dates*/
data _NULL_;
   temp=DATETIME();
   call symput('START',temp);
   call symput('STARTDATE',put(datepart(temp),date9.));
   call symput('STARTTIME',put(timepart(temp),time4.));
run;

Data _null_ ;
   Call Symput('QUERYFROMc',"&QUERYFROM.");
   Call Symput('QUERYTOc',"&QUERYTO.");      
   Call Symput('QUERYFROM',put(input("&QUERYFROM",mmddyy10.),best12.));
   Call Symput('QUERYTO',put(input("&QUERYTO",mmddyy10.),best12.));
run;

%PUT &QUERYFROM;
%PUT &QUERYTO;

data _null_;
   call symput("NUMPER",trim(left(intck("Months",&QUERYFROM.,&QUERYTO.)+1)));
   call symput("NUMYEARS",trim(left(intck('Years',&QUERYFROM.,&QUERYTO.)+1)));
   call symput("NUMMONTHS",trim(left(intck('Months',&QUERYFROM.,&QUERYTO.)+1)));
   call symput("FROMY",trim(left(year(&QUERYFROM.))));
   call symput("TOY",trim(left(year(&QUERYTO.))));
   call symput("FROMMO",trim(left(month(&QUERYFROM.))));
run;

%PUT &NUMPER.;
%PUT &NUMYEARS.;
%PUT &NUMMONTHS;
%PUT &FROMY.;
%PUT &TOY.;

data _null_;
   call symput("FORMOD",trim(left(&FROMMO.-2)));
   if month(&QUERYFROM.) = 1 then call symput("YEARCHANGE",1);
   else if month(&QUERYFROM.) = 2 then call symput("YEARCHANGE",0);
   else call symput("YEARCHANGE",14-month(&QUERYFROM.));
run; 

%PUT &YEARCHANGE.; 
%PUT &FORMOD.;

/*---------------------------------------------------------------------------*/
/*  02 -- Import / Export data                                               */
/*---------------------------------------------------------------------------*/

/********************************/
/* Importing Cports Input Files */   
/********************************/

%MACRO IMPORTFILES(var1,var2,var3);
   %IF %INDEX(%UPCASE("&VAR1."),CPORT) %THEN %DO;

proc cimport infile="&infolder.&VAR1." library=infolder memtype=data;
run;

   %END;
   %LET &VAR2.=%SUBSTR(&VAR1.,1,%INDEX(%UPCASE(&VAR1.),.)-1);
   %LET VAR1=%SUBSTR(&VAR1.,1,%INDEX(%UPCASE(&VAR1.),.)-1);

data _&var2.;
   set infolder.&var1.;
   group = compress(trim(left(group)));
   group = translate(group,'_','-',
                           '_','.',
                           '_',',',
                           '_','%',
                           '_','$',
                           '_','!',
                           '_','*',
                           '_','&',
                           '_','#',
                           '_','@');
   keep &var3.;
run;

%MEND IMPORTFILES;

%MACRO IMPORTFILES2(var1,var2);
   %IF %INDEX(%UPCASE("&VAR1."),CPORT) %THEN %DO;

proc cimport infile="&infolder.&VAR1." library=infolder memtype=data;
run;

   %END;
   %LET &VAR2.=%SUBSTR(&VAR1.,1,%INDEX(%UPCASE(&VAR1.),.)-1 );
   %LET VAR1=%SUBSTR(&VAR1.,1,%INDEX(%UPCASE(&VAR1.),.)-1);

data _&var2.;
   set infolder.&var1.;   
run;

%MEND IMPORTFILES2;
%IMPORTFILES(&QUERYFILE.,QUERYFILE,_ALL_);   
%IMPORTFILES(&QUERYEVENTFILE.,QUERYEVENTFILE,_ALL_);
   
%MACRO WRAPPER();
   %IF %INDEX(%UPCASE("&INCQUERYFILE."),.) %THEN %DO;
      %IMPORTFILES(&INCQUERYFILE.,INCQUERYFILE,Group Code CodeType SubGroup CareSettingPrincipal /*MODIF WAVE3*/ IncTrunc);
   %END;
   %ELSE %DO;

data _INCQUERYFILE;
   if 0 then set _QUERYFILE;
   /*MODIF WAVE3*/
   format IncTrunc $1.;
   IncTrunc="";
   keep Group Code CodeType SubGroup CareSettingPrincipal;
   stop;
run;

   %END;
   %IF %INDEX(%UPCASE("&INCEVENTFILE."),.) %THEN %DO;
      %IMPORTFILES(&INCEVENTFILE.,INCEVENTFILE,Group SubGroup Code Codetype CareSettingPrincipal); 
   %END;
   %ELSE %DO;

data _INCEVENTFILE;
   if 0 then set _QUERYEVENTFILE;
   keep Group SubGroup Code Codetype CareSettingPrincipal;
   stop;
run;

   %END;
   %IF %INDEX(%UPCASE("&CONDFILE."),.) %THEN %DO;
          %IMPORTFILES(&CONDFILE.,CONDFILE,Code CodeType CondFrom CondTo CareSettingPrincipal Group SubGroup Inclusion); 
   %END;
   %ELSE %DO;

data _CONDFILE;
   if 0 then set _QUERYEVENTFILE;
   format Condfrom Condto Inclusion best.;   
   condfrom=.;
   condto=.;
   Inclusion=.;
   keep Group SubGroup Codetype Code CareSettingPrincipal Condfrom Condto Inclusion;
   stop;
run;

   %END;
   %IF %INDEX(%UPCASE("&CCIFILE."),.) %THEN %DO;
         %IMPORTFILES(&CCIFILE.,CCIFILE,_ALL_); 
   %END;
   %IF %INDEX(%UPCASE("&UTILFILE."),.) %THEN %DO;
         %IMPORTFILES(&UTILFILE.,UTILFILE,_ALL_); 
   %END;
   %IF %INDEX(%UPCASE("&BMIKIDSFILE."),.) %THEN %DO;
         %IMPORTFILES(&BMIKIDSFILE.,BMIKIDSFILE,_ALL_); 
   %END;
   %IF %INDEX(%UPCASE("&OUTTABLESFILE."),.) %THEN %DO;
        %IMPORTFILES2(&OUTTABLESFILE.,OUTTABLESFILE);
   %END;

   /*MODIF WAVE3 HERE*/
   %IF %INDEX(%UPCASE("&STOCKPILINGFILE."),.) %THEN %DO;   
      %IMPORTFILES(&STOCKPILINGFILE.,STOCKPILINGFILE,_ALL_);
   %END;
   %ELSE %DO;

data _STOCKPILINGFILE;
   set _QUERYFILE(keep=Group);
   format SameDay $2. SupRange AmtRange $40. PercentDays best.;
   SameDay="aa";
   SupRange="0<-HIGH";
   AmtRange="0<-HIGH";
   PercentDays=0;   
run;

   %END;

/*MODIF WAVE3 HERE*/
proc sort nodupkey data=_STOCKPILINGFILE;
by Group;
run;

%MEND WRAPPER;
%WRAPPER();

/* Ensure CCI, UTIL and BMI modules are available*/
%macro ProcessInclude(filename=);
	%if %sysfunc(fileexist(&infolder.&filename.))=0 %then %do;
		%put ERROR: Include file &infolder.&filename. is required but does not exist;
		%abort cancel;
	%end;
	%include "&infolder.&filename.";
%mend ProcessInclude;

%MACRO WRAPPER;
   %IF %STR("&CCIFILE.") ~= %STR("") %THEN %DO;
   	  %ProcessInclude(filename=ms_cci_pre.sas);
	  %ProcessInclude(filename=ms_cci.sas);
	  %ProcessInclude(filename=ms_cci_post.sas);
	  %ProcessInclude(filename=ms_lockdata.sas);
	  %ProcessInclude(filename=ms_wordcount.sas);      
   %END;
   %IF %STR("&UTILFILE.") ~= %STR("") %THEN %DO;
   	  %ProcessInclude(filename=ms_util.sas);
   	  %ProcessInclude(filename=ms_lockdata.sas);
   	  %ProcessInclude(filename=ms_wordcount.sas);      
   %END;   
   %IF %STR("&BMIKIDSFILE.") ~= %STR("") %THEN %DO; 
	  *Ensure the Vitals table is present; 
	  %IF %SYSFUNC(exist(indata.&VITTABLE.))=0 %THEN %DO;
		  %PUT ERROR: The BMIKIDSFILE file was specified but the Vitals Table does not exist on disk.;
		  %ABORT CANCEL;
	  %END;       

	  %IF %SYSFUNC(fileexist(&infolder.ms_vdw_gc_calculate_biv.sas))=0 %THEN %DO;
		%PUT ERROR: Include file &infolder.ms_vdw_gc_calculate_biv.sas is required but does not exist;
		%ABORT CANCEL;
	  %END;

      %ProcessInclude(filename=ms_pedsbmichange.sas);	  
      %ProcessInclude(filename=ms_lockdata.sas);
      %ProcessInclude(filename=ms_wordcount.sas);
   %END;
 
%MEND WRAPPER;
%WRAPPER;

/* MODIF WAVE3 HERE */
/* Ensure Stockpiling module is available */
%ProcessInclude(filename=ms_stockpiling.sas);
/**********************************/
/* EXPORTING INPUT VALUES TO MSOC */   
/**********************************/
 
%MACRO EXPORTVALS(var1,var2);

options  nodate nonumber;
proc printto print="&MSOC.&REQUESTID.&RUNID._&var2..lst" new;
run;
Title1 "&var1. File printout";
proc print data=infolder.&var1. noobs;
run;
Title1 "";
proc printto print=print;run;

%MEND EXPORTVALS;
%EXPORTVALS(&QUERYFILE.,QUERYFILE);   
%EXPORTVALS(&QUERYEVENTFILE.,QUERYEVENTFILE);

%MACRO WRAPPER();
   %IF &INCQUERYFILE. ~= %STR() %THEN %DO;
      %EXPORTVALS(&INCQUERYFILE.,INCQUERYFILE);
   %END;
   %IF &CONDFILE. ~= %STR() %THEN %DO;
      %EXPORTVALS(&CONDFILE.,condfile); 
   %END;
   %IF &INCEVENTFILE. ~= %STR() %THEN %DO;
      %EXPORTVALS(&INCEVENTFILE.,inceventfile); 
   %END;
%MEND WRAPPER;
%WRAPPER();   

/*---------------------------------------------------------------------------*/
/*  03 -- Pre-processing of input data                                       */
/*---------------------------------------------------------------------------*/

/*MODIF WAVE3 HERE*/
/*Query*/
data _QUERYFILE;
set _QUERYFILE;

   code = compress(code,'. ');
   CodeType=upcase(CodeType);

   format WashTyp $4.;
   if CohortDef="01" then WashTyp='SING';
   else WashTyp='MULT';

   if WashPer = .  then WashTyp='MIN';
   if WashPer = .  then WashPer = 0;   
      
   if missing(EnrDays) then EnrDays=0;
run;

proc sort nodupkey data=_QUERYFILE;
   by Group codetype Code CareSettingPrincipal;
run;   

/*In some cases, we may wish that each query group in the query file be incident to 
  a common list of drugs, in this case, the list should only be entered once while leaving the
  group variable empty*/
data _INCQUERYFILE;
   set _INCQUERYFILE(rename=(Group=IGroup));
   if IGroup = "" then IGroup = "_ALLGROUPS_";   
   code = compress(code,'. ');
   CodeType=upcase(CodeType);
run;

proc sort nodupkey data=_INCQUERYFILE;
   by IGroup codetype Code CareSettingPrincipal;
run;

proc sql noprint;
create table _querycodes as
/*PART 1: if IGROUP is _ALLGROUPS_ then join the files using CodeType and Code and 
     keeping all INCQUERY codes*/
select quer.group,
       inqu.igroup,
       quer.subgroup,
       inqu.subgroup as isubgroup,
       quer.Code,
       inqu.Code as icode,
       quer.CodeType,
       inqu.CodeType as iCodeType,
	   quer.CareSettingPrincipal,
	   inqu.CareSettingPrincipal as iCareSettingPrincipal
   from _QUERYFILE as quer right join _INCQUERYFILE as inqu
   on quer.Code = inqu.Code and quer.CodeType = inqu.CodeType and 
   	  quer.CareSettingPrincipal = inqu.CareSettingPrincipal
   where IGroup = "_ALLGROUPS_"
union
/*PART 2: if IGROUP is not _ALLGROUPS_ then join the files using CodeType, Code and 
     Group to get all common codes*/
select quer.group,
       inqu.igroup,
       quer.subgroup,
       inqu.subgroup as isubgroup,
       quer.Code,
       inqu.Code as icode,
       quer.CodeType,
       inqu.CodeType as iCodeType,
	   quer.CareSettingPrincipal,
	   inqu.CareSettingPrincipal as iCareSettingPrincipal
   from _QUERYFILE as quer join _INCQUERYFILE as inqu
   on quer.Code = inqu.Code and quer.CodeType = inqu.CodeType and quer.Group = inqu.IGroup and 
   	  quer.CareSettingPrincipal = inqu.CareSettingPrincipal
   where IGroup ne "_ALLGROUPS_"
union
/*PART 3: Add QUERY only codes*/
(
   select group,
           "" as igroup,
           subgroup,
          "" as isubgroup,
           Code,
          "" as icode,
           CodeType,
          "" as iCodeType,
		   CareSettingPrincipal,
		  "" as iCareSettingPrincipal		   
   from _QUERYFILE
   where (CodeType||Code||CareSettingPrincipal) not in 
        (select (CodeType||Code||CareSettingPrincipal) from _INCQUERYFILE where IGroup = "_ALLGROUPS_")
   except
   select igroup as group,
           "" as igroup,
           subgroup,
          "" as isubgroup,
           Code,
          "" as icode,
           CodeType,
          "" as iCodeType,
		   CareSettingPrincipal,
		  "" as iCareSettingPrincipal
          from _INCQUERYFILE
)
union
/*PART 4: Add INCQUERY only codes that doesn't apply to _ALLGROUPS_*/
(
   select "" as group,
           igroup,
           "" as subgroup,
          subgroup as isubgroup,
           "" as Code,
          Code as icode,
           "" as CodeType,
          CodeType as iCodeType,
		   "" as CareSettingPrincipal,
		   CareSettingPrincipal as iCareSettingPrincipal
   from _INCQUERYFILE
   where IGroup ne "_ALLGROUPS_"
   except
   select "" as group,
           group as igroup,
           "" as subgroup,
          subgroup as isubgroup,
           "" as Code,
          Code as icode,
           "" as CodeType,
          CodeType as iCodeType,
		   "" as CareSettingPrincipal,
		   CareSettingPrincipal as iCareSettingPrincipal
          from _QUERYFILE
)
   order by Group;
quit;

data _querycodes;
   set _querycodes;

   Query = 0;
   Incid = 0;

   if Group ne "" then Query = 1;
   if IGroup ne "" then Incid = 1;

   if SubGroup = "" then SubGroup = ISubGroup;
 
   if Code = "" then do;
       Code = iCode;
	   CodeType = iCodeType;
	   CareSettingPrincipal = iCareSettingPrincipal;       
   end;  

   keep Group IGroup SubGroup Code CodeType CareSettingPrincipal Query Incid;
run;

/*MODIF WAVE3*/
proc sort data=_QueryCodes;
by IGroup codetype Code CareSettingPrincipal;
run;

/*MODIF WAVE3*/
data _QueryCodes;
merge _QueryCodes _INCQUERYFILE(in=b drop=SubGroup);
by IGroup codetype Code CareSettingPrincipal;
if not b then IncTrunc="";
run;

/*In the following steps, we retrieve the query group settings to be applied in creation 
    of episodes loop*/
proc sort nodupkey data=_QUERYFILE
                    out=_querysettings(keep=Group /*MODIF WAVE3*/ CohortDef WashTyp WashPer EpisodeGap ExpExtPer MinEpisDur 
                                            MinDaySupp EnrDays);
   by Group;
run;

/*Since a claim can either be query or incident we cannot apply at the retrieval a query group setting,
  we shall apply the most severe washout setting*/
data _extractsettings (keep=WashTyp WashPer WashOrder);
   set _querysettings(keep=WashTyp WashPer);       
   WashOrder=1;
   if strip(upcase(WashTyp))="MIN" then WashOrder=0;
run;
proc sort data = _extractsettings;
   by WashOrder descending WashPer;
run;
data _extractsettings;
   set _extractsettings;
   if _N_ = 1;
run;

data _querycodes;
   set _querycodes;
   if _N_ = 1 then set _extractsettings;
run;

/*Events*/
data _QUERYEVENTFILE;
set _QUERYEVENTFILE;
   code = compress(code,'. ');
   CodeType=upcase(CodeType);  
run;   
   
proc sort nodupkey data=_QUERYEVENTFILE;
   by Group CodeType Code CareSettingPrincipal;
run;

proc sort nodupkey data=_INCEVENTFILE;
   by Group CodeType Code CareSettingPrincipal;
run;

data _eventcodes;
   merge _QUERYEVENTFILE(in=a keep=Group SubGroup CodeType Code CareSettingPrincipal)
         _INCEVENTFILE(in=b keep=Group SubGroup CodeType Code CareSettingPrincipal);
   by Group CodeType Code CareSettingPrincipal;
   if a then event = 1; else event = 0;
   if b then incevent = 1; else incevent = 0;
run;

proc sort nodupkey data = _QUERYEVENTFILE
                    out = _eventsettings(keep=Group WashPer BlackoutPer /*MODIF WAVE3*/ EventCount);
   by Group;
run;

data _eventsettings;
/*MODIF WAVE3*/
   set _eventsettings;
   merge _eventsettings _querysettings(keep=Group CohortDef);
   by Group;
   format WashTyp $4.;

   if WashPer = .  then WashTyp='MIN';
   else if CohortDef="03" then WashTyp='SING';
   else WashTyp='MULT';

   if WashPer = .  then WashPer = 0; 

   drop CohortDef;
run;

data _eventcodes;
   merge _eventcodes(in=a) _eventsettings;
   by group;
   if a;   
run;

/*Pre-existing conditions*/
data _CONDFILE;
   retain Group codetype Code;
   set _CONDFILE;
   length CondLookTyp $2.;
   format WashTyp $4.;
   WashTyp='MULT';
   if CondFrom=. and CondTo=. then delete;  /*Defensive coding - should have one and/or the other*/
   CondLookTyp='F';
   if missing(CondFrom) then CondFrom=0;    /*Default value will ultimately include index date(=0)*/
   if missing(CondTo) then CondTo=0;        /*Default value will ultimately include index date(=0)*/
run;
proc sort nodupkey data=_CONDFILE(keep=Group SubGroup Codetype Code CareSettingPrincipal
                                 CondLookTyp WashTyp Condfrom Condto Inclusion);
    by _ALL_; /*the retain included in the previous step makes sure Group codetype Code are in this order*/
 run;

 data _diag
      _proc
      _ndc(drop=CareSettingPrincipal);
   length caresettingprincipal $ 30;
   set _querycodes(in=a) _eventcodes(in=b) _condfile(in=c);

   if not a then do;
      Query = 0;
      Incid = 0;
	  /*MODIF WAVE3*/
	  IncTrunc="";
   end;
   if not b then do;
      Event = 0;
      Incevent = 0;
   end;

   if c then cond = 1;else cond = 0;

   Code=compress(Code,'. ');
   length=length(code);
   if WashTyp = '' then WashTyp = 'MULT'; 
   if WashPer = .  then WashPer = 0;

   if CondFrom = . then CondFrom = 0;  
   if CondTo = . then CondTo = 0;

   * Translate the missing value (dot) into an underscore and the wildcard (star) into an A to avoid problems 
     with SAS functions further;
   CareSettingPrincipal = translate(CareSettingPrincipal,'A','*','_','.');
   CareSettingPrincipal = upcase(CareSettingPrincipal);

   CodeType=upcase(CodeType);

   if CodeType in:('DX') then do; codetype=compress(codetype,'DX'); output _diag; end;
   if CodeType in:('RX') then do; codetype=compress(codetype,'RX'); output _ndc; end;
   if CodeType in:('PX') then do; codetype=compress(codetype,'PX'); output _proc; end;   
   
run;

/*Storing all QueryGroup into a macro vector*/
proc sort nodupkey data = _querycodes(where=(Query=1))
                    out = _GroupList(keep=Group);
   by Group; 
run;

proc sql noprint;
select Group
   into :GROUPVECT1 separated by ' '
   from _GroupList;
quit;
proc sql noprint;
select Group
   into :GROUPVECT2 separated by ','
   from _GroupList;
quit;

%PUT &GROUPVECT1;
%PUT &GROUPVECT2;

/*Reading proc and diag codes into vectors*/
%GLOBAL VECT_proc;
%GLOBAL VECT_diag;
%MACRO CREATEVECT(file);
   %IF %SYSFUNC(exist(_&file.))=1 %THEN %DO;
data _&file.;
set  _&file.;
OrigCode=code; 
exact=index(code,'*')=0;
if exact=1 then do;
	code=compress(code,'*');
end;
else do;
	WildcardIndex=index(code,'*');
	if WildcardIndex=length(code) then do;   	/*the star is at the end*/
		code=compress(code,'*'); 
	end;
	else do;                            		/*the star is in a middle position*/
		exact=-1;		
		LengthCodeEnd=length(code)-WildcardIndex;
		CodeEnd=substr(code,WildcardIndex+1,LengthCodeEnd);		
		code=substr(code,1,WildcardIndex-1);
	end;
end;
length=length(Code); 
run;

data _null_;
   call symput("VECT_&file.","");
run;   
data _temp(keep=code);
   set  _&file.;
   format code $8.;
   code = "'"||trim(left(code))||"'";
run;
proc sql noprint;
select unique code
   into :VECT_&file. separated by ' ' 
   from _temp;
quit;

      %PUT &&VECT_&file..;
   %END;
%MEND CREATEVECT;
%CREATEVECT(proc);
%CREATEVECT(diag);

/*---------------------------------------------------------------------------*/
/* 04 -- Extract medical (diagnosis and procedures) claims                   */
/*---------------------------------------------------------------------------*/

%MACRO GETMEDS();

 proc contents data=_proc noprint out=_tilt_ ;
 data _null_ ;
    set _tilt_ ;
    call symput('pnobs',trim(left(put(nobs,15.)))) ;
 run;

   %PUT &pnobs.;

   %IF %EVAL(&pnobs.>0) %THEN %DO;

proc sql noprint;
Create Table _procextract as
select PatID, ADate, EncType, 'S' as PDX, compress(PX,'.') as PX, PX_CodeType 
   from indata.&proctable. (where=(compress(PX,'.') in:(&VECT_proc.)))
/*MODIF WAVE3: removed to allow multiple claims per day
   group by PatID, ADate, EncType, PDX, PX, PX_CodeType 
   having freq(patid) ge 1*/;
quit;

proc sql noprint;
Create Table _procedures as
Select  prctb.PatId, 
        prctb.Adate, 
        prctb.EncType, 
        prctb.pdx, 
   	    compress(prctb.PX,'. ') as Code, 
        proclist.GROUP,   
        proclist.IGroup,
        proclist.SUBGROUP,        
        proclist.CODETYPE, 
        proclist.Query, 
        proclist.Incid,   
/*MODIF WAVE3*/
		proclist.IncTrunc,
        proclist.WASHTYP, 
        proclist.WASHPER, 
        proclist.BlackoutPer, 
		proclist.EventCount,
        proclist.Event,
        proclist.IncEvent,         
        proclist.CareSettingPrincipal,
        proclist.CondLookTyp, 
        proclist.Condfrom, 
        proclist.Condto, 
        proclist.Cond, 
        proclist.Inclusion,    
        1 as RxSup,
/*MODIF WAVE3*/
        1 as RxAmt,
        1 as proc
   From _procextract as prctb,
        _proc as proclist
   Where prctb.PX_codetype = proclist.codetype and 
   		 (
         	(proclist.exact=-1 and substr(compress(prctb.PX,'.'),1,proclist.length) = proclist.code 
			  				   and substr(compress(prctb.PX,'.'),proclist.WildcardIndex+1,proclist.LengthCodeEnd) = proclist.CodeEnd) or
	      	(proclist.exact=0  and substr(compress(prctb.PX,'.'),1,proclist.length) = proclist.Code) or
          	(proclist.exact=1  and compress(prctb.PX,'.')=proclist.code)
          ) and
          prctb.Adate >= &QUERYFROM. - proclist.WashPer + proclist.CondFrom - (proclist.washtyp='MIN')*999999       
/*MODIF WAVE3: removed to allow multiple event claims per day
   group by Patid, ADate, EncType, PDX, GROUP,   IGroup,   SUBGROUP, CODE,   CODETYPE, Query, Incid,   
                WASHTYP, WASHPER, BlackoutPer, Event, IncEvent, CareSettingPrincipal, CondLookTyp,
                Condfrom, Condto, Cond, RxSup, RxAmt, proc, inclusion 
   having FREQ(ADate) GE 1*/
	Order by PatId;

   /*The 999999 is to extend the washout period long enough in the case where washtyp = 'MIN'*/
   /*The group by is to eliminate all duplicates*/
quit;
      
Title1 "Procedure code frequency printout";
proc freq data = _procedures;
   tables Code;
run;
Title1 "";

   %END;

proc contents data=_diag noprint out=_tilt_ ;
data _null_ ;
   set _tilt_ ;
   call symput('dnobs',trim(left(put(nobs,15.))));
run;

   %PUT &dnobs.;

   %IF %EVAL(&dnobs.>0) %THEN %DO;         

proc sql noprint;
Create Table _diagextract as
select PatID, ADate, EncType, PDX, compress(DX,'.') as DX, DX_CodeType 
   from indata.&diatable. (where=(compress(DX,'.') in:(&VECT_diag.)))
/*MODIF WAVE3: removed to allow multiple claims per day
   group by PatID, ADate, EncType, PDX, DX, DX_CodeType 
   having freq(patid) ge 1*/;
quit;

proc sql noprint;
Create Table _Diagnosis as
Select  diagtb.PatId, 
        diagtb.Adate, 
        diagtb.EncType, 
        diagtb.pdx, 		  
		compress(diagtb.DX,'. ') as Code,
        diaglist.GROUP,   
        diaglist.IGroup,
        diaglist.SUBGROUP,       
        diaglist.CODETYPE, 
        diaglist.Query, 
        diaglist.Incid, 
/*MODIF WAVE3*/
		diaglist.IncTrunc, 
        diaglist.WASHTYP, 
        diaglist.WASHPER, 
        diaglist.BlackoutPer,
	    diaglist.EventCount, 
        diaglist.Event,
        diaglist.IncEvent,         
        diaglist.CareSettingPrincipal,
        diaglist.CondLookTyp, 
        diaglist.Condfrom, 
        diaglist.Condto, 
        diaglist.Cond, 
        diaglist.Inclusion, 
		1 as RxSup,
/*MODIF WAVE3*/
       	1 as RxAmt,  
        -1 as proc
   From _diagextract as diagtb,
        _diag as diaglist
   Where diagtb.DX_codetype = diaglist.codetype and 
   		 (
         	(diaglist.exact=-1 and substr(compress(diagtb.DX,'.'),1,diaglist.length) = diaglist.code 
			 				   and substr(compress(diagtb.DX,'.'),diaglist.WildcardIndex+1,diaglist.LengthCodeEnd) = diaglist.CodeEnd) or
	      	(diaglist.exact=0  and substr(compress(diagtb.DX,'.'),1,diaglist.length) = diaglist.Code) or
          	(diaglist.exact=1  and compress(diagtb.DX,'.')=diaglist.code)
         ) and
         diagtb.Adate >= &QUERYFROM. - diaglist.WashPer + diaglist.CondFrom - (diaglist.washtyp='MIN')*999999           
/*MODIF WAVE3: removed to allow multiple claims per day
   group by Patid, ADate, EncType, PDX, GROUP,   IGroup,   SUBGROUP, CODE,   CODETYPE, Query, Incid,
                WASHTYP, WASHPER, BlackoutPer, Event, IncEvent, CareSettingPrincipal, CondLookTyp,
                Condfrom, Condto, Cond, proc, inclusion 
   having FREQ(ADate) GE 1*/
	Order by PatId;

   /*The 999999 is to extend the washout period long enough in the case where washtyp = 'MIN'*/
   /*The group by is to eliminate all duplicates*/
quit;

Title1 "Diagnosis code frequency printout";
proc freq data = _diagnosis;
   tables Code;
         run;
Title1 "";

   %END;

   %IF %EVAL(&pnobs.> 0 and &dnobs.>0) %THEN %DO;

data _MasterQueryfile;
   set _procedures _diagnosis;
   by PatId;
   Clm = _N_;
run;
proc datasets library = work nolist nowarn;
   delete _procedures _diagnosis _procextract _diagextract;
quit; 

   %END;
   %IF %EVAL(&pnobs.> 0 and &dnobs.=0) %THEN %DO;

data _MasterQueryfile;
   set _procedures;
   by PatId;
   Clm = _N_;
run;
proc datasets library = work nolist nowarn;
   delete _procedures _procextract;
quit;

   %END;
   %IF %EVAL(&pnobs.=0 and &dnobs.>0) %THEN %DO;

data _MasterQueryfile;
   set _diagnosis;
   by PatId;
   Clm = _N_;
run;
proc datasets library = work nolist nowarn;
   delete _diagnosis _diagextract;
quit;

   %END;

%MEND GETMEDS;
%GETMEDS();

/*---------------------------------------------------------------------------*/
/*  05 -- Envelope outpatient diagnoses during inpatient stays               */
/*---------------------------------------------------------------------------*/

/****************************************************************/
/*  Reduce diagnosis table according to selected care setting   */
/*   after having reclassified as inpatient all selected claims */
/*   within admission and discharge dates of an inpatient       */
/*   stay, when necessary                                       */
/****************************************************************/

%MACRO ENVEL();                                      

   %IF %SYSFUNC(exist(_MasterQueryfile))=1 %THEN %DO;

proc contents data=_MasterQueryfile noprint out=_tilt_ ;
data _null_ ;
   set _tilt_ ;
   call symput('nobs1',trim(left(put(nobs,15.)))) ;
run;

   %END;
   %ELSE %DO;

data _null_ ;
   call symput('nobs1',input('0',15.)) ;
run;

   %END;

   %LET SOME=0;

proc sort nodupkey data=_diag out=_diagCS(keep=CareSettingPrincipal);
   by CareSettingPrincipal;
run;

proc sort nodupkey data=_proc out=_procCS(keep=CareSettingPrincipal);
   by CareSettingPrincipal;
run;
      
data _null_;
   set _procCS _diagCS;       
      
   CareSettingPrincipal=upcase(compress(CareSettingPrincipal,"'"));

   /*first determine whether the "all care settings" option was always chosen*/
   CIP=1;
   CED=1;
   CIS=1;
   CAV=1;
   COA=1;


* Verify wildcard for EncType
  Skipping the if block means there is a wildcard for Enctype and all PDX are specified, or that all caresetting and PDX are wanted (empty variable),
  so no envelope is needed;
if not ((indexw(CareSettingPrincipal,'AAP') & indexw(CareSettingPrincipal,'AAS') & indexw(CareSettingPrincipal,'AAX') & indexw(CareSettingPrincipal,'AA_')) or 
         indexw(CareSettingPrincipal,'AAA') or CareSettingPrincipal eq "") then do;	

   CIP=0;
   CED=0;
   CIS=0;
   CAV=0;
   COA=0;

   * All combination present or wildcard used for PDX;
   if (indexw(CareSettingPrincipal,'IPP') & indexw(CareSettingPrincipal,'IPS') & indexw(CareSettingPrincipal,'IPX') & indexw(CareSettingPrincipal,'IP_')) or 
	   indexw(CareSettingPrincipal,'IPA') then CIP=1;

   * All combination present or wildcard used for PDX;
   if (indexw(CareSettingPrincipal,'EDP') & indexw(CareSettingPrincipal,'EDS') & indexw(CareSettingPrincipal,'EDX') & indexw(CareSettingPrincipal,'ED_')) or 
	   indexw(CareSettingPrincipal,'EDA') then CED=1;

   * All combination present or wildcard used for PDX;
   if (indexw(CareSettingPrincipal,'ISP') & indexw(CareSettingPrincipal,'ISS') & indexw(CareSettingPrincipal,'ISX') & indexw(CareSettingPrincipal,'IS_')) or 
	   indexw(CareSettingPrincipal,'ISA') then CIS=1;

   * All combination present or wildcard used for PDX;
   if (indexw(CareSettingPrincipal,'AVP') & indexw(CareSettingPrincipal,'AVS') & indexw(CareSettingPrincipal,'AVX') & indexw(CareSettingPrincipal,'AV_')) or 
	   indexw(CareSettingPrincipal,'AVA') then CAV=1;

   * All combination present or wildcard used for PDX;
   if (indexw(CareSettingPrincipal,'OAP') & indexw(CareSettingPrincipal,'OAS') & indexw(CareSettingPrincipal,'OAX') & indexw(CareSettingPrincipal,'OA_')) or 
	   indexw(CareSettingPrincipal,'OAA') then COA=1;
end;
   
   if CareSettingPrincipal ne '' and not(CIP & CIS & CED & CAV & COA)
      then call symput('SOME',input('1',1.));   /*this will trigger the envelope to run*/
   else CareSettingPrincipal='';      
run;

   %PUT &SOME.;
   %PUT %EVAL(&SOME.=1);
   
   %IF %EVAL(&nobs1.>0 and &SOME.=1) %THEN %DO;

/*Reclassification as inpatient all selected claims within admission and discharge dates 
    of an inpatient stay*/
/*Get Unique ADate-DDate combinations*/
proc sort nodupkey data=indata.&ENCTABLE.(keep=PatId Adate Ddate EncType where=(EncType='IP')) 
                    out=_IPdates(keep=PatId Adate Ddate);
   by PatId Adate Ddate;
run;         

proc sql noprint;
create table _datematch as
Select claimtb.Clm 
   From _MasterQueryFile(where=(EncType not in('IP'))) as claimtb,
        _IPDates as datetb
   Where claimtb.PatId = datetb.PatId and 
         datetb.ADate <= claimtb.ADate <=  max(datetb.ADate,datetb.DDate);
quit;

proc sort nodupkey data = _datematch(keep=Clm);
   by Clm;
run;

/*For each record in Diagnosis matching an inpatient day date, recode encounter type as inpatient*/
data _MasterQueryFile(drop=EncType);
   merge _MasterQueryFile(in=a) _datematch(in=b);
   by Clm;
   EncType2=EncType;
   if b then do;
      if upcase(EncType) not in('IP') then pdx='S';
      EncType2='IP';            
   end;
run;         

proc datasets library = work nolist;
   modify _MasterQueryFile;
   rename EncType2=EncType;
quit;

proc datasets library = work nolist nowarn;
   delete _IPDates;
quit;

   %END;

   %IF %SYSFUNC(exist(_MasterQueryfile))=1 %THEN %DO;

/*Filter claims with matching EncType and PDX status*/
data _MasterQueryFile(drop=EncType2 PDX2 clm CareSettingPrincipal numarg i EncType PDX /*MODIF WAVE3: needed below: code codetype*/);
   set _MasterQueryFile;
   format EncType2 $2. PDX2 $1.;
   /*case where all care settings are wanted*/
   if CareSettingPrincipal = '' then output;
   else do;
      Numarg=length(compress(CareSettingPrincipal," '"))/3;
      do i=1 to Numarg;	  	 
         EncType2=substr(compress(scan(CareSettingPrincipal,i),"'"),1,2);
		 PDX2=substr(compress(scan(CareSettingPrincipal,i),"'"),3,1);		 
		 if PDX2="_" then PDX2="";	* the underscore was used for missing PDX;

		 if proc=1 then PDX2="A";

         if EncType2 = EncType and PDX2 = PDX or 
			EncType2 = EncType and PDX2 = "A" or		/* A was used as the wildcard for PDX*/
			EncType2 = "AA" and PDX2 = PDX or			/* AA was used as the wildcard for EncType*/
			EncType2 = "AA" and PDX2 = "A" then do;	
            output;            
         end;
      end;
   end;
run;

   %END;

%MEND ENVEL;
%ENVEL();

/*---------------------------------------------------------------------------*/
/*  06 -- Extract drug claims                                                */
/*---------------------------------------------------------------------------*/

%MACRO GETDRUGS();

   %LET ISCode9=0;
   %LET ISCode11=0;

data _null_;
   set _ndc;
   if length=9 then call symput("ISCode9",1);
   if length=11 then call symput("ISCode11",1);
run;

   %PUT &ISCode9;
   %PUT &ISCode11;

   /*Extract 9 and/or 11 digit Codes claims*/
   %MACRO WRAPPER;
      %IF %EVAL(&ISCode9.>0) %THEN %DO;

proc sql noprint;
   create table _drugs as
   select CodeList.*, 
          Dispensing.Patid, 
          Dispensing.RxDate as ADate,
          Dispensing.RxSup,
          Dispensing.RxAmt, 
          0 as Proc
   from _ndc as CodeList,
        indata.&DISTABLE. as Dispensing
   where RxSup > 0 and
         substr(Dispensing.ndc,1,9) = CodeList.code and Dispensing.Rxdate >= 
         &QUERYFROM. - CodeList.WashPer + CodeList.CondFrom - 
         (upcase(CodeList.washtyp)='MIN')*999999;
quit;

/*MODIF WAVE3: removed to allow multiple claims per day
proc sort nodupkey data = _drugs;
   by _ALL_;
run;            
*/

Title1 "Nine digit NDC code frequency printout";
proc freq data = _drugs;
   tables Code;
run;
Title1 "";   

      %END;
      %IF %EVAL(&ISCode11.>0) %THEN %DO;

proc sql noprint;
create table _predrugs as
select CodeList.*,
       Dispensing.Patid, 
       Dispensing.RxDate as ADate,
       Dispensing.RxSup, 
       Dispensing.RxAmt,
       0 as Proc
   from _ndc as CodeList,
        indata.&DISTABLE. as Dispensing
   where RxSup > 0 and 
         substr(Dispensing.ndc,1,11) = CodeList.code and Dispensing.Rxdate >= 
         &QUERYFROM. - CodeList.WashPer + CodeList.CondFrom - 
         (upcase(CodeList.washtyp)='MIN')*999999;
quit;

/*MODIF WAVE3: removed to allow multiple claims per day
proc sort nodupkey data = _predrugs;
   by _ALL_;
run; 
*/ 

Title1 "Eleven digit NDC code frequency printout";
proc freq data = _predrugs;
   tables Code;
run;
Title1 "";

proc datasets library=work nolist nowarn;
   append base=_drugs data=_predrugs FORCE;
   delete _predrugs;
quit;

      %END;

   %MEND WRAPPER;
   %WRAPPER;

%MEND GETDRUGS;
%GETDRUGS();

%MACRO WRAPPER;
   %IF %SYSFUNC(exist(_MasterQueryFile))=1 and %SYSFUNC(exist(_Drugs))=1 %THEN %DO;

data _MasterQueryFile;
format code $11.;
   set _MasterQueryFile _drugs/*MODIF WAVE3 (drop=code codetype)*/;
run;  
proc datasets library = work nolist nowarn;
   delete _drugs;
quit; 

   %END;
   %IF %SYSFUNC(exist(_MasterQueryFile))=0 and %SYSFUNC(exist(_Drugs))=1 %THEN %DO;

data _MasterQueryFile;
format code $11.;
   set _drugs/*MODIF WAVE3 (drop=code codetype)*/;
run;  
proc datasets library = work nolist nowarn;
   delete _drugs;
quit;     

   %END;
%MEND WRAPPER;
%WRAPPER;

/*Verify if at least one claim was extracted from the CDM databases*/
%macro wrapper();
	%if %sysfunc(exist(_MasterQueryFile))=0 %then %do;
		%put ERROR: No members meet cohort inclusion/exclusion criteria for all groups;
		%abort cancel;
	%end;	
%mend wrapper;
%wrapper;



/*MODIF WAVE3*/
* Eliminate same day duplicates. For events check what to do with the EventCount parameter; 
data _premaster1 _premaster2 _premaster3;
set _MasterQueryFile;
if event and EventCount=2 then output _premaster1;
else if (event and EventCount=1) or cond or incevent then output _premaster2;
else output _premaster3;
run;

proc sort nodupkey data=_premaster1;
by Group PatId ADate;
run;

proc sort nodupkey data=_premaster2;
by Group PatId ADate CodeType Code Cond Event IncEvent;
run;

data _MasterQueryFile;
set _premaster1 _premaster2 _premaster3;
drop CodeType Code;
run;


/*---------------------------------------------------------------------------*/
/*  07 -- Stockpile drugs                                                    */
/*---------------------------------------------------------------------------*/

data _MasterQueryFile(keep=PatId ADate Group IGroup SubGroup Query Incid RxSup RxAmt /*MODIF WAVE3*/ Proc IncTrunc) 
/*MODIF WAVE3*/
_MasterEventFile(keep=PatId ADate Group IGroup SubGroup WashTyp WashPer BlackoutPer Event IncEvent RxSup RxAmt proc) 
_MasterCondFile(keep=PatId ADate Group IGroup SubGroup WashTyp WashPer CondLookTyp CondFrom CondTo Cond proc RxSup RxAmt Inclusion);
   set _MasterQueryFile;
   if query or incid then output _MasterQueryFile;
   else if event or incevent then output _MasterEventFile;
   else output _MasterCondFile; 
run;

/*MODIF WAVE3 HERE*/
%macro STOCKPILE(configfile=,file=,whereQ=,whereI=,class=,id=);

	data _null_;
		set _GroupList end=fin;
		if fin then call symput('NQUERYGROUPS',_n_);
	run;

	%put &NQUERYGROUPS.;

  	%do i = 1 %to &NQUERYGROUPS.;

		data _null_;
		   set _GroupList;
		   if _n_ = &i. then call symput('ITGROUP',strip(Group));
		run;       

		%if %str("&configfile.") ne %str("") %then %do;
			data _null_;
			set &configfile.;
			if Group in("&ITGROUP.") then do;
			  call symput('SAMEDAY',strip(SameDay));
			  call symput('SUPRANGE',strip(SupRange)); 
			  call symput('AMTRANGE',strip(AmtRange)); 
			  call symput('PERCENTDAYS',PercentDays);
			end;   
			run;
		%end;
		%else %do;
			%let SAMEDAY=aa;
			%let SUPRANGE=0<-HIGH;
		    %let AMTRANGE=0<-HIGH;
		    %let PERCENTDAYS=;
		%end;

		%let PERCENTDAYS = %trim(&PERCENTDAYS.);
		%if %str("&PERCENTDAYS.")=%str("0") %then %do;
			%let PERCENTDAYS=;
		%end;

		%put &ITGROUP. &SAMEDAY. &SUPRANGE. &AMTRANGE. &PERCENTDAYS.;

		*QUERY records (Group non blank);
		data &file.Q_iter;
		set &file.;
		where &whereQ. and Group in("&ITGROUP.");
		run;

		%MS_STOCKPILING(INFILE=&file.Q_iter,	  		
		                CLMDATE=ADate,                            		
		                CLMSUP=RxSup,                              		
		                CLMAMT=RxAmt,	                           		
					    PROCFLAG=proc,                                 		
					    PERCENTDAYS=&PERCENTDAYS.,                          		
					    GROUPING=group &class. subgroup,                   		
					    SAMEDAY=&SAMEDAY.,                                 	
						SUPRANGE=&SUPRANGE.,									
						AMTRANGE=&AMTRANGE.,									
					    ID=IGroup &id.,                                       		
					    OUTFILE=&file.Q_iter,  		
						OUTFILEEXCL=&file.Q_iterex 	
		);

		data &file.Q_iterex;
		set &file.Q_iterex;
		format Group $30.;
		Group="&ITGROUP.";
		run;		
		
	  	%if &I.=1 %then %do;
			data &file.Q;
			   set &file.Q_iter;
			run;			 
			data &file.Qex;
			   set &file.Q_iterex;
			run;
		%end;
		%else %do;
			proc append base = &file.Q
			            data = &file.Q_iter force;
			run;		
			proc append base = &file.Qex
			            data = &file.Q_iterex force;
			run;		
		%end;
	%end;

	*INCID only records (Group blank and IGroup non blank);	
	data &file.I;
	set &file.;
	where &whereI.;
	run;

	*Stockpiling input file only applies to QUERYFILE codes;
	%MS_STOCKPILING(INFILE=&file.I,	  		
	                CLMDATE=ADate,                            		
	                CLMSUP=RxSup,                              		
	                CLMAMT=RxAmt,	                           		
				    PROCFLAG=proc,                                 		
				    PERCENTDAYS=,                          		
				    GROUPING=igroup &class. subgroup,                   		
				    SAMEDAY=aa,                                 	
					SUPRANGE=0<-HIGH,									
					AMTRANGE=0<-HIGH,									
				    ID=Group &id.,                                       		
				    OUTFILE=&file.I,  		
					OUTFILEEXCL=&file.Iex 	
	);

%mend STOCKPILE;
%STOCKPILE(configfile=_stockpilingfile,file=_MasterQueryFile,whereQ=Query,whereI=not Query and Incid,class=IncTrunc,id=Query Incid);
%STOCKPILE(configfile=_stockpilingfile,file=_MasterEventFile,whereQ=Event,whereI=not Event and IncEvent,class=,id=Event IncEvent WashPer WashTyp BlackoutPer);
%STOCKPILE(configfile=_stockpilingfile,file=_MasterCondFile,whereQ=Cond,whereI=not Cond,class=CondFrom CondTo Inclusion,id=Cond WashPer WashTyp CondLookTyp);

%macro STOCKPILE_EXCLUDED(infile=,claimtype=);
	proc means data=&infile. nway noprint;
	var ExclSup ExclAmt ExclBoth;
	class Group PatId;
	id Patient;
	output out=&infile.(drop=_:) sum(ExclSup ExclAmt ExclBoth)=
								 N(ExclSup)=ExclClaims;
	run;

	proc means data=&infile. nway noprint;
	var Patient ExclClaims ExclSup ExclAmt ExclBoth;	
	class Group;
	output out=&infile.(drop=_:) N(Patient)=ExclMembers
								 sum(ExclClaims ExclSup ExclAmt ExclBoth)=;
	run;

	data &infile.;
	retain Group ClaimType ExclMembers ExclClaims ExclSup ExclAmt ExclBoth;
	merge &infile.(in=a) _grouplist(in=b);
	by Group;
	format ClaimType $10.;
	ClaimType="&claimtype.";
	if a then Order=0;
	else if b then do;
		Order=1;
		ExclMembers=0;
		ExclClaims=0;
		ExclSup=0;
		ExclAmt=0;
		ExclBoth=0;
	end;
	run;
%mend STOCKPILE_EXCLUDED;
%STOCKPILE_EXCLUDED(infile=_masterqueryfileqex,claimtype=QUERY);
%STOCKPILE_EXCLUDED(infile=_mastereventfileqex,claimtype=EVENT);
%STOCKPILE_EXCLUDED(infile=_mastercondfileqex,claimtype=CONDITION);

data msoc.&REQUESTID.&RUNID._stockpiling_excl;
set _masterqueryfileqex _mastereventfileqex _mastercondfileqex;
run;

proc sort data=msoc.&REQUESTID.&RUNID._stockpiling_excl;
by Group ClaimType Order;
run;

proc sort nodupkey data=msoc.&REQUESTID.&RUNID._stockpiling_excl 
				   out=msoc.&REQUESTID.&RUNID._stockpiling_excl(drop=Order);
by Group ClaimType;
run;

/*---------------------------------------------------------------------------*/
/*  08 -- Extract eligible patients                                          */
/*---------------------------------------------------------------------------*/

Proc SQL Noprint;
Create Table _DenomInt as
Select Demogs.PatId,
       Demogs.Birth_Date,
       Demogs.sex,
       Enrol.Enr_Start,
       Enrol.Enr_End,
	   CASE
			WHEN upcase(MEDCOV) eq 'Y'  THEN 1 ELSE 0
	   END as MedCov,
	   CASE
			WHEN upcase(DRUGCOV) eq 'Y'  THEN 1 ELSE 0
	   END as DrugCov
   From indata.&ENRTABLE.(where=(not missing(DRUGCOV) and not missing(MEDCOV))) as Enrol,
        indata.&DEMTABLE.(where=(Birth_Date ne . and not missing(PatId) and not missing(Sex))) as Demogs
   Where Demogs.PatId=Enrol.PatId 
   order by Patid, Enr_Start, Enr_end;
Quit;

%macro wrapper();
	data _DenomInt;
	   set  _DenomInt(where=(Enr_End >= Enr_Start));
	   by PatId;
	   format MinAgeDate MaxAgeDate Enr_End Enr_Start mmddyy10.;

	   MinAgeDate = intnx(scan("&AGETYP.",1),birth_date,&MINAGE.,'sameday');

	   if &MAXAGE.=99999 then MaxAgeDate=intnx('Years',birth_date,110,'sameday');
	   else MaxAgeDate= intnx(scan("&AGETYP.",&NUMAGECAT.),birth_date,&MAXAGE.+1,'sameday')-1; 

	   *Change episode if maximum allowable gap is reached or if changes occur in benefit coverage;
		%IF %UPCASE("&COVERAGE.") eq %STR("MD") %THEN %DO; 
			if Enr_Start-lag(Enr_end)-1 > &ENROLGAP. or MedCov ne lag(MedCov) or DrugCov ne lag(DrugCov) then episode=episode+1; 
		%END;
		%IF %UPCASE("&COVERAGE.") eq %STR("M") %THEN %DO; 
			if Enr_Start-lag(Enr_end)-1 > &ENROLGAP. or MedCov ne lag(MedCov) then episode=episode+1;
		%END;
		%IF %UPCASE("&COVERAGE.") eq %STR("D") %THEN %DO; 	
			if Enr_Start-lag(Enr_end)-1 > &ENROLGAP. or DrugCov ne lag(DrugCov) then episode=episode+1;
		%END;
		if first.Patid then episode=1;
	   retain episode;
	run;
%mend wrapper;
%wrapper();

/*Reconciliation of eligible episodes*/
proc means data=_DenomInt nway noprint;
   var Enr_Start Enr_end;
   class PatId episode;
   id Birth_Date sex MinAgeDate MaxAgeDate MedCov DrugCov;
   output out=_DenomInt(drop=_:) min(Enr_Start)= max(Enr_end)=;
run;

/*Ensure all claims kept are in patients enrollment period*/
proc sql noprint;
create table _MasterQueryFile as 
select mqfq.*
   from _MasterQueryFileQ as mqfq,
        _DenomInt as den
   where mqfq.PatId=den.PatId and den.Enr_Start <= mqfq.ADate <= den.Enr_End
union all corr 
select mqfi.*
   from _MasterQueryFileI as mqfi,
        _DenomInt as den
   where mqfi.PatId=den.PatId and den.Enr_Start <= mqfi.ADate <= den.Enr_End;
quit;

/*MODIF WAVE3*/
data _MasterNonQueryFile0;
set _mastereventfileq
	_mastereventfilei
	_mastercondfileq 
	_mastercondfilei;
run;

proc sql noprint;
create table _MasterNonQueryFile as 
select mqfq.*
   from _MasterNonQueryFile0 as mqfq,
        _DenomInt as den
   where mqfq.PatId=den.PatId and den.Enr_Start <= mqfq.ADate <= den.Enr_End;
quit;

proc datasets library = work nolist nowarn;
/*MODIF WAVE3*/
   delete _MasterQueryFileQ _MasterQueryFileI _MasterNonQueryFile0
		  _MasterQueryFileQEx _MasterQueryFileQ_Iter _MasterQueryFileQ_IterEx _MasterQueryFileIEx
		  _MasterEventFileQ _MasterEventFileI _MasterEventFile
		  _MasterEventFileQEx _MasterEventFileQ_Iter _MasterEventFileQ_IterEx _MasterEventFileIEx
		  _MasterCondFileQ _MasterCondFileI _MasterCondFile
		  _MasterCondFileQEx _MasterCondFileQ_Iter _MasterCondFileQ_IterEx _MasterCondFileIEx;
quit;

data _DenomInt;
set _DenomInt;

TopCount = 0;
if Enr_End >= &QUERYFROM. and Enr_Start <= &QUERYTO. then TopCount = 1; 

AgeCount = 0;
if TopCount and (MaxAgeDate >= Enr_Start and MinAgeDate <= Enr_End) then AgeCount = 1;

run;

proc sql noprint;
create table _Coverage as 
select PatId,
	   MedCov,
       DrugCov,
	   max(MedCov+DrugCov) as MaxNumCov 
from _DenomInt
where TopCount=1
group by PatId;
quit;

proc means data=_Coverage nway noprint;
var MedCov DrugCov;
class PatId;
where MaxNumCov < 2;
output out=_Coverage(drop=_:) min=;
run;

data _Coverage;
set _Coverage;
*Flag that indicates if a patient has at least one episode with both Drug and Medical coverage or 
 if he always has the same coverage pattern for all episodes;
PatHasBothCovOROneCovPatternDum=1;
*This case means that we have at leat one episode without MedCov and at least one without DrugCov;
if MedCov=0 and DrugCov=0 then PatHasBothCovOROneCovPatternDum=0;	
run;

data _DenomInt;
merge _DenomInt _Coverage(in=b keep=PatId PatHasBothCovOROneCovPatternDum);
by PatId;
if not b then PatHasBothCovOROneCovPatternDum=1;
MedCovDum=MedCov;
DrugCovDum=DrugCov;
run;

%MACRO WRAPPER;
   %IF %UPCASE("&COVERAGE.") eq %STR("MD") or %UPCASE("&COVERAGE.") eq %STR("DM") %THEN %DO;
   	  data _DenomInt;
	  set _DenomInt;
	  if PatHasBothCovOROneCovPatternDum=0 then do;
	  	MedCovDum=1;
		DrugCovDum=1;
	  end;
	  run;
   %END;
   %IF %UPCASE("&COVERAGE.") eq %STR("M") %THEN %DO;
      data _DenomInt;
	  set _DenomInt;	  
	  PatHasBothCovOROneCovPatternDum=1;
	  DrugCovDum=1;	 
	  run;
   %END;
   %IF %UPCASE("&COVERAGE.") eq %STR("D") %THEN %DO;
      data _DenomInt;
	  set _DenomInt;	  
	  PatHasBothCovOROneCovPatternDum=1;
	  MedCovDum=1;	 
	  run;
   %END;   
%MEND WRAPPER;
%WRAPPER;

proc sql noprint;
create table _waterfall_ ( 
	CritNum num format=best., 
	CritDesc char(200) format=$200., 
	CritCnt num format=best.);
quit;

%MS_ATTRITIONTABLE(INFILE=_DenomInt,CRITFLAG=TopCount,CLASSVARS=PatId,WHERE=1,EXCLUDEDVAR=,
                   OUTFILE=_waterfall_,
                   CRITDESC="Initial Member Count - Members with a non-missing birth date/sex at any enrollment episode 
 overlapping the query period",
                   CRITNUM=1,
                   CRITCNTNAME=CritCnt,
                   ADDITIONALVARS=
);

%MS_ATTRITIONTABLE(INFILE=_DenomInt,CRITFLAG=DrugCovDum,CLASSVARS=PatId,WHERE=TopCount=1,EXCLUDEDVAR=,
                   OUTFILE=_waterfall_,
                   CRITDESC="Exclusion  Members must be excluded if they only have episodes with DrugCov=N and MedCov=Y during the query period",
                   CRITNUM=2,
                   CRITCNTNAME=CritCnt,
                   ADDITIONALVARS=
);

%MS_ATTRITIONTABLE(INFILE=_DenomInt,CRITFLAG=MedCovDum,CLASSVARS=PatId,WHERE=TopCount=1 and DrugCovDum=1,EXCLUDEDVAR=,
                   OUTFILE=_waterfall_,
                   CRITDESC="Exclusion  Members must be excluded if they only have episodes with DrugCov=Y and MedCov=N during the query period",
                   CRITNUM=3,
                   CRITCNTNAME=CritCnt,
                   ADDITIONALVARS=
);

%MS_ATTRITIONTABLE(INFILE=_DenomInt,CRITFLAG=PatHasBothCovOROneCovPatternDum,CLASSVARS=PatId,WHERE=TopCount=1 and DrugCovDum=1 and MedCovDum=1,EXCLUDEDVAR=,
                   OUTFILE=_waterfall_,
                   CRITDESC="Exclusion  Members must be excluded if they only have episodes with DrugCov=Y and MedCov=N and DrugCov=N and MedCov=Y during the query period",
                   CRITNUM=4,
                   CRITCNTNAME=CritCnt,
                   ADDITIONALVARS=
);

%MS_ATTRITIONTABLE(INFILE=_DenomInt,CRITFLAG=AgeCount,CLASSVARS=PatId,WHERE=TopCount=1 and DrugCovDum=1 and MedCovDum=1 and PatHasBothCovOROneCovPatternDum=1,EXCLUDEDVAR=,
                   OUTFILE=_waterfall_,
                   CRITDESC="Exclusion - Members must satisfy the age range condition within the query period",
                   CRITNUM=5,
                   CRITCNTNAME=CritCnt,
                   ADDITIONALVARS=
);

proc sql noprint;
create table _waterfall as 
select grp.Group,
       wat.CritNum,
	   wat.CritDesc,
	   wat.CritCnt
   from _waterfall_ as wat,
        _grouplist as grp;
quit;

data _DenomInt;
   set _DenomInt(where=(DrugCovDum=1 and MedCovDum=1 and PatHasBothCovOROneCovPatternDum=1));
   
   if enr_end < MinAgeDate then delete;

   LastAgeGroup =&NUMAGECAT.;
   do i=&NUMAGECAT. to 1 by -1;
      Threshdate=intnx(scan("&AGETYP.",i),birth_date,scan("&AGETHRESH.",i),'sameday'); 
      if  Threshdate<= min(&QUERYTO.,Enr_end) then do;
         LastAgeGroup =i;
         FirstAgeGroup=i;
         if Threshdate > max(&QUERYFROM.,Enr_Start)   then do;
            do j=i-1 to 1 by -1;
               if intnx(scan("&AGETYP.",j),birth_date,scan("&AGETHRESH.",j),'sameday') <= 
                    max(&QUERYFROM.,Enr_Start)  then do;
                  FirstAgeGroup=j;
                  leave;
               end;
            end;
         end;
         leave;
      end; 
   end;
   rename episode=EligEpisode;
   drop i j threshdate TopCount AgeCount DrugCov MedCov DrugCovDum MedCovDum PatHasBothCovOROneCovPatternDum;
run;

/**********************/
/* Create _FdateTable */
/**********************/

proc means noprint data =_MasterQueryFile nway;
   var ADate;
   Class PatId Group;
   where &QUERYFROM. <= ADate <= &QUERYTO. and Query=1;
   output out=_FDateTable(drop=_:) min = MinDt;
run;

    /*Create QueryGroup Date Variables*/
proc transpose data = _FDateTable
                out = _FDateTable(drop=_NAME_);
   id Group;
   var MinDt;
   by PatId;
run;   

proc transpose data=_querysettings out=_Groups(drop=_NAME_);
id Group;
run;

data _Groups;
set _Groups;
if _N_ = 0;
run;


data _FDateTable;
   set _FDateTable;
   if _N_=0 then set _Groups;
   format MinFdt MMDDYY10. MaxFdt MMDDYY10.;
   MinFdt = min(&GROUPVECT2.,.);
   MaxFdt = max(&GROUPVECT2.,.);
run;

/*Reduce enrollment to Query Patients*/
data _Enrollment(keep=PatId Enr_Start Enr_End sex birth_date MinAgeDate MaxAgeDate);
   if 0 then set _FDateTable;
   declare hash ht (hashexp:16, dataset:'_FDateTable');
   ht.definekey('PatId');
   ht.definedata(ALL: 'YES');
   ht.definedone();

   do until(eof1);
      set _DenomInt end=eof1;
      if ht.find()=0 then do;
         output _Enrollment;
      end;
   end;
   stop;
run;

/*Because CCI or UTIL are required (by the presence of input files), 
    a sub_sample will be generated from potential patients for multiple access of their
    complete history*/
%MACRO WRAPPER;
   %IF %STR("&CCIFILE.") ~= %STR("") %THEN %DO;   	  
      %MS_LOCKDATA(INFILE=indata.&DIATABLE.(where=(DX_Codetype='09')),
                   PATLISTINFILE=_FDateTable,
                   OUTFILE=_DIACCI(drop=Provider EncounterID OrigDX EncType EncType));

      %MS_LOCKDATA(INFILE=indata.&PROCTABLE.(where=(PX_Codetype in('C4','09'))),
                   PATLISTINFILE=_FDateTable,
                   OUTFILE=_PROCCCI(drop=Provider EncounterID OrigPX EncType EncType));
   %END;
   %IF %STR("&UTILFILE.") ~= %STR("") %THEN %DO;   	  
      %MS_LOCKDATA(INFILE=indata.&ENCTABLE.,
                PATLISTINFILE=_FDateTable,
                OUTFILE=_ENCUTIL(keep=PatId ADate EncType));
   %END;   
%MEND WRAPPER;
%WRAPPER;


%LET I=1;

/*---------------------------------------------------------------------------*/
/*  09 -- Create and process episodes                                        */
/*---------------------------------------------------------------------------*/
%global NQUERYGROUPS;
%MACRO LOOPTHROUGH();

data _null_;
   set _GroupList end=fin;
   if fin then call symput('NQUERYGROUPS',_n_);
run;

   %PUT &NQUERYGROUPS.;
   %DO i = 1 %TO &NQUERYGROUPS.;

data _null_;
   set _GroupList;
   if _n_ = &i. then call symput('ITGROUP',strip(Group));
run;

      %PUT &ITGROUP.;      
      %LET EVENTWASHOUT=0;
	  %LET BLACKOUTPER=0;

data _null_;
   set _eventsettings;
   if Group in("&ITGROUP.") then do;
      call symput('EVENTWASHOUT',WashPer);
      call symput('BLACKOUTPER',BlackOutPer); 
   end;   
run;

      %PUT &EVENTWASHOUT.;

data _loopsettings;
   set _querysettings(where=(Group in("&ITGROUP.")));
   drop Group;
run;

data _SDFDateTable(keep=PatId WashTyp WashPer EpisodeGap ExpExtPer MinEpisDur MinDaySupp EnrDays);
   set _FDateTable;
   if _N_ = 1 then set _loopsettings;
   if &ITGROUP.;   
run;

      /*Load CCI and/or UTIL macro parameters for ITGROUP.  If not, leave blank.*/
      %MACRO CCIREQD;
         %IF %STR("&CCIFILE.") ~= %STR("") %THEN %DO;

data _null_;
   set _CCIFILE;
   where Group in("&ITGROUP.");
   call symput("CCIPRIORDAYS",trim(left(PRIORDAYS)));
   call symput("INDEXDAYS",trim(left(INDEXDAYS)));
   call symput("CCIINCLINDEX",trim(left(INCLINDEX)));
   call symput("CCIGROUP",trim(left(CCIGROUP)));
   call symput("KEEPALLDUM",trim(left(KEEPALLDUM)));
   run;

            %PUT &CCIPRIORDAYS. &INDEXDAYS. &CCIINCLINDEX. &CCIGROUP. &KEEPALLDUM.;
         %END;
         %ELSE %DO;

data _null_;
   call symput("CCIPRIORDAYS","");
   call symput("INDEXDAYS","");
   call symput("CCIINCLINDEX","");
   call symput("CCIGROUP","");
   call symput("KEEPALLDUM","");
run;

         %END;
      %MEND CCIREQD;
      %CCIREQD;
      %MACRO UTILREQD;
         %IF %STR("&UTILFILE.") ~= %STR("") %THEN %DO;

data _null_;
   set _UTILFILE;
   where Group in("&ITGROUP.");
   call symput("CARESETTINGS",trim(left(CARESETTINGS)));
   call symput("UTILPRIORDAYS",trim(left(PRIORDAYS)));
   call symput("UTILINCLINDEX",trim(left(INCLINDEX)));
   call symput("CSSTRAT",trim(left(CSSTRAT)));
run;

            %PUT &CARESETTINGS. &UTILPRIORDAYS. &UTILINCLINDEX. &CSSTRAT.;
         %END;
         %ELSE %DO;

data _null_;
   call symput("CARESETTINGS","");
   call symput("UTILPRIORDAYS","");
   call symput("UTILINCLINDEX","");
   call symput("CSSTRAT","");
run;

         %END;
      %MEND UTILREQD;
      %UTILREQD;
      
   %MACRO BMIKIDREQD;
     %IF %STR("&BMIKIDSFILE.") ~= %STR("") %THEN %DO; 
       data _null_;
          set _BMIKIDSFILE;
          where Group in("&ITGROUP.");
          call symput("BMIPreIndexEnd",trim(left(PreIndexEnd)));
          call symput("BMIPreDays",trim(left(PreDays)));
          call symput("BMIPostIndexStart",trim(left(PostIndexStart)));
          call symput("BMIPostDays",trim(left(PostDays)));
          call symput("BMIHtLag",trim(left(HtLag)));
          call symput("BMIGroup",trim(left(BMIGroup)));
       run;

       %PUT &BMIPreIndexEnd. &BMIPreDays. &BMIPostIndexStart. &BMIPostDays. &BMIHtLag. &BMIGroup.;
     %END;
     %ELSE %DO;
       data _null_;
          call symput("BMIPreIndexEnd","");
          call symput("BMIPreDays","");
          call symput("BMIPostIndexStart","");
          call symput("BMIPostDays","");
          call symput("BMIHtLag","");
          call symput("BMIGroup","");
       run;
     %END;
   %MEND BMIKIDREQD;
   %BMIKIDREQD;   
   
/*Select all claims for this study drug and identify those that are within query dates*/
data _Quantity /*MODIF WAVE3 (drop=query incid)*/;
   if 0 then set _SDFDateTable;
   declare hash ht (hashexp:16, dataset:'_SDFDateTable');
   ht.definekey('PatId');
   ht.definedata(ALL: 'YES');
   ht.definedone();

   do until(eof1);
      set _MasterQueryfile(where=((Group in("&ITGROUP.") or 
                   IGroup in("&ITGROUP.","_ALLGROUPS_")) and (query or incid))) end=eof1;  
      if ht.find()=0 then do;
         InQuery = 0;
         if Group in("&ITGROUP.") and &QUERYFROM. <= ADate /*MODIF WAVE3 <= &QUERYTO.*/ then InQuery = 1;
         output _Quantity; 
      end;
   end;
   stop;
run;

proc sort data = _Quantity;
   by PatId SubGroup ADate ExpireDt descending InQuery;
run;
proc sort nodupkey data = _Quantity;
   by PatId SubGroup ADate ExpireDt;
run;

/*Defensive coding, enroll episodes do not overlap*/
Proc SQL Noprint;
Create Table _CleanQuantity as
Select qty.*,
       enrol.Enr_Start,
       enrol.Enr_End,
       enrol.MinAgeDate,
       enrol.MaxAgeDate
   From _Quantity as qty,
        _Enrollment as enrol
   Where qty.PatId = enrol.PatId and enrol.Enr_Start <= qty.ADate <= enrol.Enr_End
        Order by PatId, ADate;
quit;

proc datasets library=work nolist nowarn;
   delete _quantity;
quit;

/*Episode creation*/
data _CleanQuantityEp;
   set _CleanQuantity (/*MODIF WAVE3*/ where=(Query and InQuery));
   by PatId;

   /*Create Episode Indicator*/
   LRunOutDate = lag(Expiredt);  
   /*MODIF WAVE3 LInQuery = lag(InQuery);*/
   LEnrStart = lag(Enr_Start);

   if first.PatId then do;
      LRunOutDate=.;
      /*MODIF WAVE3 LInQuery=.;*/
      LEnrStart=.; 
      episode=1;
      gap=.;
   end;
   else do; 
      gap = Adate - LRunOutDate-1;   
      /*MODIF WAVE3 diff = InQuery - LInQuery;*/
      if gap > EpisodeGap /*MODIF WAVE3 or diff in(-1,1)*/ or lEnrStart ne Enr_Start then episode=episode+1; 
      /*change episode if exceed gap (above)*/
   end;
   
   Group = "&ITGROUP.";   

   retain episode;
run;

/*Create episode summary for all episode, including those InQuery=0 to further capture next 
      episode start date*/
proc means data=_CleanQuantityEp nway noprint;
   var Expiredt Adate RxSup /*MODIF WAVE3*/ RxAmt /*MODIF WAVE3 NoclaimBf */ Enr_Start Enr_End;
   class PatId episode;
   id InQuery Group WashTyp Washper MinDaySupp MinEpisDur ExpExtPer EnrDays MinAgeDate MaxAgeDate;
   output out=_EPisodeSummary(drop=_Type_ rename=_freq_=NumDispensing) 
              min(Adate Enr_Start Enr_End)=EpisodeStartDt MinEnrStart MinEnrEnd
              max(Expiredt /*MODIF WAVE3 NoclaimBf */ Enr_Start Enr_End)=EpisodeEndDt /*MODIF WAVE3 NoclaimBf */ MaxEnrStart MaxEnrEnd
              sum(RxSup RxAmt)=EpisodeRxSup EpisodeRxAmt;
run;

/* MODIF WAVE3 New Block Start */
proc sql noprint;
Create Table _NoClaimBf as
Select    epis.PatId,
          epis.Episode,
          epis.EpisodeStartDt,
          epis.WashTyp,
          epis.WashPer,
          qty.Expiredt
   From _CleanQuantity as qty 
   right join
        _EPisodeSummary as epis         
   on    qty.PatId = epis.PatId 
   where qty.ADate < epis.EpisodeStartDt                  
   order by PatId, episode;
quit;

data _NoClaimBf;
set _NoClaimBf;
NoClaimBf = 1;
if WashTyp in('MIN') then NoClaimBf = 0;
else if WashTyp in('MULT','SING') then do;
   if (EpisodeStartDt - ExpireDt - 1) < WashPer then NoClaimBf = 0; 
end;
run;

proc means noprint data = _NoClaimBf nway;
var NoClaimBf;
class PatId episode;
output out=_NoClaimBf(drop=_:) min= ;
run;

proc sql noprint;
create table _IncQueryDate as 
select qty.PatId,
	   min(qty.ADate) as IncQueryDate format mmddyy10.,
	   ep.Episode 
	from _CleanQuantity as qty,
		 _EpisodeSummary as ep
	where qty.PatId=ep.PatId and qty.Incid=1 and upcase(IncTrunc)="Y" and
		  ep.EpisodeStartDt <= qty.ADate <= ep.EpisodeEndDt
	group by qty.PatId, Episode;
quit;
/* MODIF WAVE3 New Block End */

data _NextDate;
   set _EPisodeSummary;
   episode=episode-1;
   rename EpisodeStartDt=NextEpisodeStartDt;
   keep patid episode EpisodeStartDt;
run;

data _ValidEpisodes;
   merge _EPisodeSummary(in=a) _NextDate /*MODIF WAVE3*/_NoClaimBf(in=b) _IncQueryDate;
   by patid episode;
   if a; 

   /*MODIF WAVE3*/
   if not b then NoClaimBf = 1;

   /*MODIF WAVE3*/
   EpisodeRxAmt = floor(EpisodeRxAmt);

   EnrNumDays= EpisodeStartDt - MinEnrStart;
   Enr = 0;
   if EnrNumDays>=0 and EnrNumDays >= EnrDays then Enr = 1;

   /*Washout. Take the max of query washout and event washout*/   
   if EnrNumDays>=0 and EnrNumDays >= max(WashPer,&EVENTWASHOUT.) then Wash = 1; *In case the washout is greater than EnrDays;
   else Wash = 0;

   /*Identify if episode is incident or not*/
   if NoclaimBf = 1 and Wash and Enr = 1 then Incident = 1;
   else Incident = 0;          

   /*episode truncation if necessary*/
   EpisodeEndDt=min(EpisodeEndDt+ExpExtPer,NextEpisodeStartDt-1,MaxEnrEnd,/*MODIF WAVE3*/IncQueryDate-1);
   
   /*keeping only valid episodes*/
   if MinAgeDate <= EpisodeStartDt <= MaxAgeDate;
   /*MODIF WAVE3*/
   if &QUERYFROM. <= EpisodeStartDt <= &QUERYTO.;
run;

/*MODIF WAVE3 HERE*/
*Adjust episodes claims information;
proc sort data=_cleanquantityep;
by PatID Episode;
run;

data _cleanquantityep2;
merge _cleanquantityep _validepisodes(in=b keep=PatId Episode EpisodeEndDt);
by PatId Episode;
if b;
if ADate > EpisodeEndDt then do;
	NumDispensing=0;
	EpisodeRxSup=0;
	EpisodeRxAmt=0;
end;
else do;
	NumDispensing=1;
	if ExpireDt > EpisodeEndDt then EpisodeRxSup=RxSup-(ExpireDt-EpisodeEndDt);
	else EpisodeRxSup=RxSup;
	if ExpireDt > EpisodeEndDt then EpisodeRxAmt=RxAmt*(RxSup-(ExpireDt-EpisodeEndDt))/RxSup;
	else EpisodeRxAmt=RxAmt;
end;
run;

proc means data=_cleanquantityep2 nway noprint;
var EpisodeRxSup EpisodeRxAmt NumDispensing;
class PatId Episode;
output out=_cleanquantityep2(drop=_:) sum=;
run;

data _ValidEpisodes;
merge _ValidEpisodes _cleanquantityep2;
by PatId Episode;
format PCHRLSON CHRLSON XCHRLSON NumVisit $10. BMIz_Strat $20.;

/*episode duration and minimum drug use*/
if (EpisodeRxSup >= MinDaySupp) and (EpisodeEndDt - EpisodeStartDt + 1) >= MinEpisDur then Duration=1;
else Duration=0;

GClaim = 1;

MinDaySuppDum = 0;
if EpisodeRxSup >= MinDaySupp then MinDaySuppDum = 1;

MinEpisDurDum = 0;
if MinDaySuppDum and (EpisodeEndDt - EpisodeStartDt + 1) >= MinEpisDur then MinEpisDurDum = 1;

BlackoutDum = 0;
if (MinDaySuppDum and MinEpisDurDum) and (EpisodeStartDt + &BLACKOUTPER.) <= EpisodeEndDt then BlackoutDum = 1;

QueryEnrDum = 0;
if (MinDaySuppDum and MinEpisDurDum and BlackoutDum) and Enr then QueryEnrDum = 1;

QueryWashDum = 0;
if (MinDaySuppDum and MinEpisDurDum and BlackoutDum and QueryEnrDum) and EnrNumDays >= WashPer then QueryWashDum = 1;

EventWashDum = 0; 
if (MinDaySuppDum and MinEpisDurDum and BlackoutDum and QueryEnrDum and QueryWashDum) and EnrNumDays >= &EVENTWASHOUT. then EventWashDum = 1;

IncQueryDum = 0;
if (MinDaySuppDum and MinEpisDurDum and BlackoutDum and QueryEnrDum and QueryWashDum and EventWashDum) and NoClaimBf then IncQueryDum = 1;

   PCHRLSON=""; 
   CHRLSON=""; 
   XCHRLSON=""; 
   NumVisit="";

   BMIPct_Pre   =.;
   BMIPct_Post  =.;
   BMIPct_Change=.;
   BMIz_Pre     =.;
   BMIz_Post    =.;
   BMIz_Change  =.;
   BMIz_Strat	="";

   keep Patid Episode EpisodeStartDt EpisodeEndDt NumDispensing EpisodeRxSup /*MODIF WAVE3*/ EpisodeRxAmt ExpExtPer Group WashTyp 
        /*EnrNumdays required for assessing eligibility criteria in the presence of exclusions*/
        EnrNumDays PCHRLSON CHRLSON XCHRLSON NumVisit 
        BMIPct_Pre BMIPct_Post BMIPct_Change BMIz_Pre BMIz_Post BMIz_Change BMIz_Strat  
        InQuery Incident Duration GClaim MinDaySuppDum MinEpisDurDum BlackoutDum QueryEnrDum QueryWashDum EventWashDum IncQueryDum;
run;


%MS_ATTRITIONTABLE(INFILE=_ValidEpisodes,CRITFLAG=GClaim,CLASSVARS=PatId,WHERE=1,EXCLUDEDVAR=,
                   OUTFILE=_waterfall,
                   CRITDESC="Exclusion - Members must have at least one QUERYGROUP claim within the query period",
                   CRITNUM=6,
                   CRITCNTNAME=CritCnt,
                   ADDITIONALVARS=Group="&ITGROUP."
);

%MS_ATTRITIONTABLE(INFILE=_ValidEpisodes,CRITFLAG=MinDaySuppDum,CLASSVARS=PatId,WHERE=1,EXCLUDEDVAR=,
                   OUTFILE=_waterfall,
                   CRITDESC="Exclusion - Members must have at least one QUERYGROUP episode with at least minimum days supplied",
                   CRITNUM=7,
                   CRITCNTNAME=CritCnt,
                   ADDITIONALVARS=Group="&ITGROUP."
);

%MS_ATTRITIONTABLE(INFILE=_ValidEpisodes,CRITFLAG=MinEpisDurDum,CLASSVARS=PatId,WHERE=1,EXCLUDEDVAR=,
                   OUTFILE=_waterfall,
                   CRITDESC="Exclusion - Members must have at least one QUERYGROUP episode with at least minimum days duration",
                   CRITNUM=8,
                   CRITCNTNAME=CritCnt,
                   ADDITIONALVARS=Group="&ITGROUP."
);

%MS_ATTRITIONTABLE(INFILE=_ValidEpisodes,CRITFLAG=BlackoutDum,CLASSVARS=PatId,WHERE=1,EXCLUDEDVAR=,
                   OUTFILE=_waterfall,
                   CRITDESC="Exclusion - Members must have at least one QUERYGROUP episode with more than blackout days duration",
                   CRITNUM=9,
                   CRITCNTNAME=CritCnt,
                   ADDITIONALVARS=Group="&ITGROUP."
);

%MS_ATTRITIONTABLE(INFILE=_ValidEpisodes,CRITFLAG=QueryEnrDum,CLASSVARS=PatId,WHERE=1,EXCLUDEDVAR=,
                   OUTFILE=_waterfall,
                   CRITDESC=". Exclusion - Members must have at least one QUERYGROUP episode satisfying the enrollment criterion specified by the 'EnrDays' parameter",
                   CRITNUM=10,
                   CRITCNTNAME=CritCnt,
                   ADDITIONALVARS=Group="&ITGROUP."
);

%MS_ATTRITIONTABLE(INFILE=_ValidEpisodes,CRITFLAG=QueryWashDum,CLASSVARS=PatId,WHERE=1,EXCLUDEDVAR=,
                   OUTFILE=_waterfall,
                   CRITDESC="Exclusion - Members must have at least one QUERYGROUP episode satisfying the enrollment criterion specified by the Query 'WashPer' parameter",
                   CRITNUM=11,
                   CRITCNTNAME=CritCnt,
                   ADDITIONALVARS=Group="&ITGROUP."
);

%MS_ATTRITIONTABLE(INFILE=_ValidEpisodes,CRITFLAG=EventWashDum,CLASSVARS=PatId,WHERE=1,EXCLUDEDVAR=,
                   OUTFILE=_waterfall,
                   CRITDESC="Exclusion - Members must have at least one QUERYGROUP episode satisfying the enrollment criterion specified by the Event 'WashPer' parameter",
                   CRITNUM=12,
                   CRITCNTNAME=CritCnt,
                   ADDITIONALVARS=Group="&ITGROUP."
);

%MS_ATTRITIONTABLE(INFILE=_ValidEpisodes,CRITFLAG=IncQueryDum,CLASSVARS=PatId,WHERE=1,EXCLUDEDVAR=,
                   OUTFILE=_waterfall,
                   CRITDESC="Exclusion - Members must have at least one QUERYGROUP episode that meets Query incidence criterion [no QUERYGROUP claim in the prior Query 'WashPer' days]",
                   CRITNUM=13,
                   CRITCNTNAME=CritCnt,
                   ADDITIONALVARS=Group="&ITGROUP."
);

data _ValidEpisodes;
set _ValidEpisodes;

if InQuery and Incident and Duration and BlackoutDum;

drop InQuery Incident Duration GClaim MinDaySuppDum MinEpisDurDum QueryEnrDum QueryWashDum EventWashDum IncQueryDum BlackoutDum;
run;

/*---------------------------------------------------------------------------*/
/*  10 -- Determine Charlson, Utilities                                      */
/*---------------------------------------------------------------------------*/

         /************************************************************************/
         /* NOTE: looping method will be used to avoid claim duplication when   */
         /* patients have more than one valid episode (MULT or SING)             */
         /************************************************************************/

         /************************************************************/
         /* Macro for preparing to run multiple cci and/or util     */
         /* (loop-through in case of MULT or SING).                   */
         /************************************************************/
           
         /* This macro will be used by BMIKids as well without modifications made */
        
      %MACRO cci_util_prep;

/*Create one record per patient*/
proc transpose data=_ValidEpisodes out=_patid_dates(keep=Patid epi:) prefix=epi;
   var EpisodeStartDt;
   by Patid;
   id episode;
run;

/*Get episode list*/
proc contents data=_patid_dates noprint out=_varlist VARNUM;
   run;

/*Since sorted by NAME, resort by VARNUM*/
proc sort data=_varlist; 
   by VARNUM;
   where upcase(NAME) ne "PATID"; 
run;

/*identify first and last episode number*/
data _varlist;
   set _varlist end=eof1;
   if _N_=1 then call symput("FIRSTVAR",strip(NAME));
   if eof1 then do;
      call symput("LASTVAR",strip(NAME));
      call symput("NUMEPIVAR",strip(put(VARNUM-1,best.)));
   end;
run;
         /*looping parameters*/
         %PUT &FIRSTVAR. &LASTVAR. &NUMEPIVAR.;
      %MEND cci_util_prep;

         /*******************************************/
         /* Macro calculating CCIs for each episode */
         /*******************************************/

      %MACRO RUNCCI;

         /* here we create a subset of the subset so that we can loop on it many times
              in the case of MULT -  the subset is created only for this ITGROUP*/
         %MS_LOCKDATA(INFILE=_DIACCI(where=(DX_Codetype='09')),
                         PATLISTINFILE=_patid_dates,
                         OUTFILE=_DIACCI&i.);

         %MS_LOCKDATA(INFILE=_PROCCCI(where=(PX_Codetype in('C4','09'))),
                         PATLISTINFILE=_patid_dates,
                         OUTFILE=_PROCCCI&i.);

         %LET epinum=1;

         %DO EPINUM=1 %TO &NUMEPIVAR.;

/*get "real" episode number*/
data _null_;
   set _varlist;
   where VARNUM=&EPINUM.+1;
   call symput("EPISODE",strip(compress(NAME,'epi','lu')));
run;

            %PUT &EPISODE.;

/*keep patients from this episode*/
data _epi&EPINUM.;
   set _patid_dates;
   array V(*)  &FIRSTVAR.--&LASTVAR.;
   format refdt mmddyy10.;
   refdt=V(&EPINUM.);
   if refdt ne .;
   keep Patid refdt;
run;

/*check Eligibility (patients without eligibility will have missing values in their CCIs)*/
data _epi&EPINUM.;
   merge _epi&EPINUM.(in=a) _ValidEpisodes(keep=PatId EpisodeStartDt EnrNumDays rename=EpisodeStartDt=refdt);
   by patid refdt;
   if a;
   if EnrNumDays>=&CCIPRIORDAYS.;
run;

            /*compute cci for this episode*/
            %ms_cci_pre(PRIORDAYS=&CCIPRIORDAYS.,
                        INDEXDAYS=&INDEXDAYS.,
                        INCLINDEX=&CCIINCLINDEX.,
                        INFILE=_epi&EPINUM.,
                        INDEXDT=refdt,
                        PREOUTFILE=_charlson&EPINUM.,
                        DIAGFILE=_DIACCI&i.,
                        PROCFILE=_PROCCCI&i.
                        );

            %ms_cci(SETIN=_charlson&EPINUM.,
                       PATID=PatId,
                       IDXPRI=IDXPRI,
                       DXVARSTR=ICDDX,
                       NDXVAR=1,
                       SXVARSTR=ICDPX,
                       NSXVAR=1,
                       HCPCS=HCPCS,
                       DSOUT=_cci&EPINUM.
                       );

            %ms_cci_post(DSIN=_cci&EPINUM.,
                            INDEXFILE=_epi&EPINUM.,
                            OUTFILE=_cci&EPINUM.,
                            CCIGROUP=&CCIGROUP., 
                            KEEPALLDUM=0
                       );

            /*add back episode number info*/
data _cci&EPINUM.;
   retain patid episode;
   set _cci&EPINUM.;
   episode=input("&EPISODE.",best12.);
run;

            /*to avoid warnings*/
            %IF &EPINUM.=1 %THEN %DO;

data _cci;
   set _cci&EPINUM.;
run;
proc datasets library=work nolist nowarn;
   delete _cci&EPINUM. _charlson&EPINUM.;
quit;

            %END;
            %ELSE %DO;

/*set-by so that the final dataset is sorted*/
data _cci;
   set _cci _cci&EPINUM.;
   by PatId episode;
run;
/*delete interim files*/
proc datasets library=work nolist nowarn;
   delete _cci&EPINUM. _charlson&EPINUM. ;
quit;

            %END; 
         %END;

proc datasets library=work nowarn nolist nowarn;
   delete _DIACCI&i. _PROCCCI&i.;
quit;

      %MEND RUNCCI;


/********************************************/
/* Macro calculating UTILs for each episode */
/********************************************/


      %MACRO RUNUTIL;

         /* here we create a subset of the subset so that we can loop on it many times
              in the case of MULT -  the subset is created only for this ITGROUP*/
         %MS_LOCKDATA(INFILE=_ENCUTIL,
                      PATLISTINFILE=_patid_dates,
                      OUTFILE=_ENCUTIL&i.(keep=PatId ADate EncType));


         %DO EPINUM=1 %TO &NUMEPIVAR.;

/*get "real" episode number*/
data _null_;
   set _varlist;
   where VARNUM=&EPINUM.+1;
   call symput("EPISODE",strip(compress(NAME,'epi','lu')));
run;
            %PUT &EPISODE.;

/*keep patients from this episode*/
data _epi&EPINUM.;
   set _patid_dates;
   array V(*)  &FIRSTVAR.--&LASTVAR.;
   format refdt mmddyy10.;
   refdt=V(&EPINUM.);
   if refdt ne .;
   keep Patid refdt;
run;

/*check Eligibility*/
data _epi&EPINUM.;
   merge _epi&EPINUM.(in=a) _ValidEpisodes(keep=PatId EpisodeStartDt EnrNumDays rename=EpisodeStartDt=refdt);
   by patid refdt;
   if a;
   if EnrNumDays>=&UTILPRIORDAYS.;
run;

            %MS_UTIL(ENCOUNTERFILE=_ENCUTIL&i.,
                      INDEXFILE=_epi&EPINUM.,
                      INDEXDT=refdt,
                      CARESETTINGS=&CARESETTINGS.,
                      PRIORDAYS=&UTILPRIORDAYS.,
                      INCLINDEX=&UTILINCLINDEX.,
                      CSSTRAT=&CSSTRAT.,
                      OUTFILE=_util&EPINUM.
               );

/*add back episode number info*/
data _util&EPINUM.;
   retain patid episode;
   set _util&EPINUM.;
   episode=input("&EPISODE.",best12.);
run;

            /*to avoid warnings*/
            %IF &EPINUM.=1 %THEN %DO;

data _util;
   set _util&EPINUM.;
run;

            %END;
            %ELSE %DO;

/*set-by so that the final dataset is sorted*/
data _util;
   set _util _util&EPINUM.;
   by PatId episode;
run;

            %END; 
         %END;

proc datasets library=work nowarn nolist nowarn;
   delete _ENCUTIL&i.;
quit;

      %MEND RUNUTIL;


      /* Macro calculating BMIKids for each episode */
      %MACRO RUNBMIKIDS;

         %DO EPINUM=1 %TO &NUMEPIVAR.;
           /*get "real" episode number*/
           data _null_;
              set _varlist;
              where VARNUM=&EPINUM.+1;
              call symput("EPISODE",strip(compress(NAME,'epi','lu')));
           run;
           %PUT &EPISODE.;

           /*keep patients from this episode*/
           data _epi&EPINUM.;
              set _patid_dates;
              array V(*)  &FIRSTVAR.--&LASTVAR.;
              format refdt mmddyy10.;
              refdt=V(&EPINUM.);
              if refdt ne .;
              keep Patid refdt;
           run;

           /*check Eligibility -- no such requirement built in BMI, but could be added easily if desired */
           data _epi&EPINUM.;
              merge _epi&EPINUM.(in=a) _ValidEpisodes(keep=PatId EpisodeStartDt EnrNumDays rename=EpisodeStartDt=refdt);
              by patid refdt;
              if a;
              /*if EnrNumDays>=&UTILPRIORDAYS.; */
           run;
           
           /* Create format based on BMIGroup input */           
           %macro DefineBMIFormat();	 
             %if %scan(&BMIGroup.,1) ^= 0 %then %do;
               %let BMIGroup = 0 &BMIGroup.;
             %end;
             %MS_WORDCOUNT(TXTVECTOR=&BMIGroup., WORDCOUNT=NUMGRP);
           	data _null_;
           		format FMTSTR $300.;		
           		do i=1 to &NUMGRP.;
                 if i ^= &NumGrp. then do;
                   word  = scan("&BMIGroup.", i, ' ') || " -< " || scan("&BMIGroup.",i+1, ' ') || " = '" || scan("&BMIGroup.", i, ' ') || " -< " || scan("&BMIGroup.",i+1, ' ') || "'";
                   word2 = "-" || scan("&BMIGroup.", i+1, ' ') || " -< -" || scan("&BMIGroup.",i, ' ') || " = '-" || scan("&BMIGroup.", i+1, ' ') || " -< -" || scan("&BMIGroup.",i, ' ') || "'";
                 end;
                 else do;
                   word  = scan("&BMIGroup.", i, ' ') || " - high = '" || scan("&BMIGroup.", i, ' ') || "-high'";
                   word2 = "low - -" || scan("&BMIGroup.", i, ' ') || "= 'low- -" || scan("&BMIGroup.", i, ' ') || "'";
                 end;
           			
           			FMTSTR=strip(FMTSTR) || " " || trim(left(word)) || " " || trim(left(word2));
           
           			if i = &NumGrp. then do;
           			  call symput("FMTSTR",FMTSTR);
           			end;
           		end;
           	run;
           	PROC FORMAT;
           	   VALUE BMIGroup
           		   &FMTSTR.
           	   ;
           	run;
           %mend DefineBMIFormat;
           %DefineBMIFormat();
           
           ***CALL THE BMI MACRO HERE;
           %PedsBMIChange(Infile          = _epi&EPINUM.
                        , Outfile         = _bmikids&EPINUM.
                        , IndexVar        = refdt
                        , VitalsInFile    = indata.&VITTABLE.
                        , DemogInFile     = indata.&DEMTABLE.
                        , PreIndexEnd     = &BMIPreIndexEnd.
                        , PreDays         = &BMIPreDays.
                        , PostIndexStart  = &BMIPostIndexStart.
                        , PostDays        = &BMIPostDays.
                        , ZScoreFormat    = BMIGroup
                        , PctileFormat    = ""
                        , HtLag           = &BMIHtLag.
                        );

           /*add back episode number info*/
           data _bmikids&EPINUM.;
              retain patid episode;
              set _bmikids&EPINUM.;
              episode=input("&EPISODE.",best12.);
           run;

            /*to avoid warnings*/
            %IF &EPINUM.=1 %THEN %DO;
             data _bmikids;
                set _bmikids&EPINUM.;
				BMIz_Strat=trim(left(BMIz_Strat));
             run;
            %END;
            %ELSE %DO;
              /*set-by so that the final dataset is sorted*/
              data _bmikids;
                 set _bmikids _bmikids&EPINUM.;
                 by PatId episode;
				 BMIz_Strat=trim(left(BMIz_Strat));
              run;
            %END; 
            
            proc datasets library=work nowarn nolist;
              delete _bmikids&EPINUM.;
              delete _epi&EPINUM.;
            quit;
         %END;
      %MEND RUNBMIKIDS;
      
      %MACRO EXEC;
	  	proc contents data=_validepisodes noprint out=_tilt_ ;
		data _null_ ;
		   set _tilt_ ;
		   call symput('nobsepis',trim(left(put(nobs,15.)))) ;
		run;
		/*Ensure there is at least one episode before executing tools macro*/
		%IF %EVAL(&nobsepis.>0) %THEN %DO;
	         %IF %STR("&CCIPRIORDAYS.") ~= %STR(" ") | %STR("&UTILPRIORDAYS.") ~= %STR("") | %STR("&BMIPREINDEXEND.") ~= %STR(" ") %THEN %cci_util_prep;
	         %IF %STR("&CCIPRIORDAYS.") ~= %STR(" ") %THEN %RUNCCI;
	         %IF %STR("&UTILPRIORDAYS.") ~= %STR(" ") %THEN %RUNUTIL;
	         %IF %STR("&BMIPREINDEXEND.") ~= %STR(" ") %THEN %RUNBMIKIDS;	         
		%END;
      %MEND EXEC;
      %EXEC;


/*check for empty files for later merge with _cci*/
      %GLOBAL CCIINCL;

	  %IF %SYSFUNC(exist(_cci)) = 1 %THEN %DO;

data _null_;
   dsid=open("_cci");
   call symput("CCIINCL"," ");
   if attrn(dsid,"NOBS") > 0 then call symput("CCIINCL","_cci");
run;

      %END;
      %ELSE %DO;

%LET CCIINCL = ;

      %END;

      %PUT &CCIINCL.;

/*check for empty files for later merge with _util*/
      %GLOBAL UTILINCL;

	  %IF %SYSFUNC(exist(_util)) = 1 %THEN %DO;

data _null_;
   dsid=open("_util");
   call symput("UTILINCL"," ");
   if attrn(dsid,"NOBS") > 0 then call symput("UTILINCL","_util");
run;

      %END;
	  %ELSE %DO;

%LET UTILINCL = ;

	  %END;

      %PUT &UTILINCL.;
         
/*check for empty files for later merge with _bmikids*/
      %GLOBAL BMIKIDSINCL;

	  %IF %SYSFUNC(exist(_bmikids)) = 1 %THEN %DO;

data _null_;
   dsid=open("_bmikids");
   call symput("BMIKIDSINCL"," ");
   if attrn(dsid,"NOBS") > 0 then call symput("BMIKIDSINCL","_bmikids");
run;

      %END;
	  %ELSE %DO;

%LET BMIKIDSINCL = ;

	  %END;

      %PUT &BMIKIDSINCL.;

/*for the present episodes, the missing charlson/NumVisit will be overwritten*/
data _ValidEpisodes;
   merge _ValidEpisodes(in=a) &CCIINCL. &UTILINCL. &BMIKIDSINCL.;
   by PatId episode;
run;          

      %IF &I.=1 %THEN %DO;

data DPLocal.&REQUESTID.&RUNID._ValidEpisodes;
   set _ValidEpisodes;
run;   
      %END;
      %ELSE %DO;

proc append base = DPLocal.&REQUESTID.&RUNID._ValidEpisodes
            data = _ValidEpisodes force;
run;

      %END; 
   %END;  /*NQUERYGROUPS loop end*/
%MEND LOOPTHROUGH;
%LOOPTHROUGH();

/*---------------------------------------------------------------------------*/
/*  11 -- Process claims for events and pre-existing conditions              */
/*---------------------------------------------------------------------------*/

/*Process events and pre-existing conditions*/
proc sql noprint;
create table _events_and_conds as
select episodes.PatId, 
       episodes.Group,
       episodes.episode,
       episodes.EpisodeStartDt,
       episodes.EpisodeEndDt,
       episodes.ExpExtPer,
       episodes.washtyp as EpWashTyp,
       claims.Adate,
       claims.event,
       claims.incevent,
       claims.cond,
       claims.washper,
       claims.washtyp,                  
       claims.CondLookTyp,
       claims.CondFrom,
       claims.CondTo,
       claims.inclusion,
/*MODIF WAVE3*/
	   claims.numdispensing
   from _masternonqueryfile(where=(event or incevent or cond)) as claims right join
        DPLocal.&REQUESTID.&RUNID._ValidEpisodes as episodes
      on claims.PatId=episodes.Patid and claims.Group=episodes.Group;
quit;

data _events_and_conds;
   if 0 then set _eventsettings(keep=Group BlackoutPer EventCount);
   declare hash ht (hashexp:16,dataset:"_eventsettings");
   ht.definekey('Group');
   ht.definedata(ALL:'YES');
   ht.definedone();

   do until(eof1);
      set _events_and_conds end=eof1;      
      if ht.find()=0 then output;
   end;
   stop;
run;

data _events_and_conds;
   set _events_and_conds;
   format EventDt mmddyy10.;

   HadEvent = 0;
   HadIncEvent = 0;
/*MODIF WAVE3*/
   NumEvent=0; 
 
   if (event or incevent) then do; 
      if (EpisodeStartDt-Washper-(washtyp='MIN')*999999) <= Adate < min(EpisodeStartDt+BlackoutPer,EpisodeEndDt) 
         then HadIncEvent = 1;
   end;   
   
   if event and min(EpisodeStartDt+BlackoutPer,EpisodeEndDt) <= Adate <= EpisodeEndDt then do;
      HadEvent = 1;
      EventDt = ADate;
	  NumEvent=NumDispensing;
   end;
  
   drop NumDispensing;
run;

/* For each group, determine if pre-existing conditions inclusion and/or exclusion are required*/
data _condfile;
   set _condfile;      
   InclReqd=0;
   ExclReqd=0;
   if inclusion=1 then InclReqD=1;
   if inclusion=0 then ExclReqD=1;      
run;

proc means data=_condfile nway noprint;
   var InclReqD ExclReqD;
   class Group;   
   output out= _condReqD(drop=_:) max=;
run;

proc sort data=_events_and_conds;
   by Group;
run;

data _events_and_conds;
   merge _events_and_conds _condReqD;
   by Group;      
run;

/*Identify claims overlapping pre-existing condition period*/
data _events_and_conds;
   set _events_and_conds;

   /* Must not apply pre-existing condition claims occurring too far from index dates*/      
   if cond=1 and (ADate-CondFrom<EpisodeStartDt or ADate-CondTo>EpisodeStartDt) then cond=0;
   
PIncCondDum = 0;
PExcCondDum = 1;

   /*If no inclusion are required for this group, we set the default HadCond=1*/
   if InclReqd=1 then HadCond = 0;
   else do;
      HadCond = 1;  /*only exclusions are required or no pre-existing conditions at all*/
      PIncCondDum = 1;
   end;

   if cond and Adate ne . then do;
      if EpisodeStartDt + CondFrom <= ADate <= EpisodeStartDt + CondTo then do;
         if inclusion=1 then do;
         	HadCond = 1;
            PIncCondDum = 1;
		 end;
         else do;
         	HadCond = 2;
            PExcCondDum = 0; 
		 end;
      end;
   end;
run;
      
/* Verify eligibility for exclusion pre-existing condition*/
proc sort nodupkey data=_condfile out=_CondExcl;
   by Group CondFrom CondTo;
   where inclusion=0;
run;
   
data _CondExcl;
   set _CondExcl;
   by Group;
   if first.Group then ExclNum=1;
   else ExclNum = ExclNum + 1;
   retain ExclNum;
run;         

proc means data=_CondExcl nway noprint;
   var ExclNum;
   class Group;
   output out=_CondExclTot(drop=_:) max=TotExcl;
run;

data _CondExcl;
   merge _CondExcl _CondExclTot;
   by Group;
run;

proc sql noprint;
create table _ExclEligFlags as
Select claims.Patid,
       claims.EpisodeStartDt,
       edates.Enr_Start,
       edates.Enr_End,
       lookup.Group,
       lookup.CondFrom,
       lookup.CondTo,
       lookup.ExclNum,
       lookup.TotExcl
   from _CondExcl(keep=Group CondFrom CondTo ExclNum TotExcl) as lookup,
        DPLocal.&REQUESTID.&RUNID._ValidEpisodes(keep=Group Patid EpisodeStartDt) as claims,
        _Enrollment(keep=Enr_Start Enr_End PatId) as edates
   where lookup.Group = claims.Group and claims.PatId = edates.PatId;
quit;

data _ExclEligFlags;
   set _ExclEligFlags;
   Meet=1;
   /*Enrollment episode completely overlaps this exclusion period*/
   if Enr_Start <= EpisodeStartDt + CondFrom and Enr_end >= EpisodeStartDt + CondTo then output; 
   keep Group Patid EpisodeStartDt ExclNum TotExcl Meet;  
run;

proc means data=_ExclEligFlags nway noprint;
   var Meet;
   class Group PatId EpisodeStartDt ExclNum;
   id TotExcl;
   output out=_ExclEligFlags(drop=_:) max=;
run;

proc means data=_ExclEligFlags nway noprint;
   var Meet;
   class Group PatId EpisodeStartDt;
   id TotExcl;
   output out=_ExclEligFlags(drop=_:) sum=NumExclCritMeet;
run;

/*Members who meet the eligibility criteria for all of the exclusion period*/
data _ExclEligFlags;
   set _ExclEligFlags;
   where NumExclCritMeet=TotExcl;
   keep Group PatId EpisodeStartDt;
run;

proc sort data = _events_and_conds;
   by Group PatId EpisodeStartDt descending HadCond;   
run;
proc sort nodupkey data = _ExclEligFlags;
   by Group PatId EpisodeStartDt;
run;

data _events_and_conds;
   merge _events_and_conds(in=a) _ExclEligFlags(in=b);
   by Group PatId EpisodeStartDt;
   if a;

   if  ExclReqD=1 then ExclCritMeet=0;
   else ExclCritMeet=1;

   if b then ExclCritMeet=1;
run;

/*Note:  At this step (_incconc_and_conds) and for each unique Adate, we know if the patient meets or doesn't meet 
          the INCL/EXCL criteria. Specifically, 
         - HadCond = 2 means that FROM THIS ADATE, the member has one or more claim meeting 
                        the EXCLUSION criteria (from this Adate) 
         - HadCond = 1 means that FROM THIS ADATE, the member has one or more claim meeting 
                      the INCLUSION criteria, but has no claim meeting the EXCLUSION criteria
         - HadCond = 0 means that FROM THIS ADATE, the member has no claim for neither 
                      INCLUSIONS or EXCLUSIONS
*/   

proc means noprint data = _events_and_conds nway;
   var HadEvent HadIncEvent HadCond EventDt ExclCritMeet PIncCondDum PExcCondDum;
   class PatId Group Episode;
   id EpWashTyp BlackoutPer /*MODIF WAVE3*/WashTyp;
   output out= _events_and_conds(drop=_:) min(EventDt ExclCritMeet PExcCondDum) =
                                          max(HadEvent HadIncEvent HadCond PIncCondDum)=
										  sum(NumEvent)=AllEvents;
run;

proc sort data = DPLocal.&REQUESTID.&RUNID._ValidEpisodes;
   by PatId Group Episode;
run;

data _premaster;
   merge DPLocal.&REQUESTID.&RUNID._ValidEpisodes(in=a) _events_and_conds(in=b);
   by PatId Group Episode;
   if a;

   HadEvent=HadEvent>0;            
   HadIncEvent=HadIncEvent>0;   

   if HadEvent and HadIncEvent=0 then do;
      EpisodeEndDt=EventDt;
   end;

IncEventDum = 0;
if HadIncEvent = 0 then IncEventDum = 1;

ExclEligDum = 0;
if IncEventDum and ExclCritMeet then ExclEligDum = 1;

ExcCondDum = 0;
if (IncEventDum and ExclEligDum) and PExcCondDum then ExcCondDum = 1;

IncCondDum = 0;
if (IncEventDum and ExclEligDum and ExcCondDum) and PIncCondDum then IncCondDum = 1;

run;

%macro FINALIZEATTRITION();

data _null_;
   set _GroupList end=fin;
   if fin then call symput('NQUERYGROUPS',_n_);
run;

   %PUT &NQUERYGROUPS.;
   %DO i = 1 %TO &NQUERYGROUPS.;

data _null_;
   set _GroupList;
   if _n_ = &i. then call symput('ITGROUP',strip(Group));
run;

      %PUT &ITGROUP.;

%MS_ATTRITIONTABLE(INFILE=_premaster,CRITFLAG=IncEventDum,CLASSVARS=PatId,WHERE=Group="&ITGROUP.",EXCLUDEDVAR=,
                   OUTFILE=_waterfall,
                   CRITDESC="Exclusion - Members must have at least one QUERYGROUP episode that meets Event incidence criterion [no QUERYEVENTGROUP claim in the prior Event 'WashPer' days]",
                   CRITNUM=14,
                   CRITCNTNAME=CritCnt,
                   ADDITIONALVARS=Group="&ITGROUP."
);

%MS_ATTRITIONTABLE(INFILE=_premaster,CRITFLAG=ExclEligDum,CLASSVARS=PatId,WHERE=Group="&ITGROUP.",EXCLUDEDVAR=,
                   OUTFILE=_waterfall,
                   CRITDESC="Exclusion - Members must have at least one QUERYGROUP episode satisfying the Exclusion enrollment requirement",
                   CRITNUM=15,
                   CRITCNTNAME=CritCnt,
                   ADDITIONALVARS=Group="&ITGROUP."
);

%MS_ATTRITIONTABLE(INFILE=_premaster,CRITFLAG=ExcCondDum,CLASSVARS=PatId,WHERE=Group="&ITGROUP.",EXCLUDEDVAR=,
                   OUTFILE=_waterfall,
                   CRITDESC="Exclusion - Members must have at least one QUERYGROUP episode satisfying the Exclusion conditions",
                   CRITNUM=16,
                   CRITCNTNAME=CritCnt,
                   ADDITIONALVARS=Group="&ITGROUP."
);

%MS_ATTRITIONTABLE(INFILE=_premaster,CRITFLAG=IncCondDum,CLASSVARS=PatId,WHERE=Group="&ITGROUP.",EXCLUDEDVAR=,
                   OUTFILE=_waterfall,
                   CRITDESC="Exclusion - Members must have at least one QUERYGROUP episode satisfying the Inclusion conditions",
                   CRITNUM=17,
                   CRITCNTNAME=CritCnt,
                   ADDITIONALVARS=Group="&ITGROUP."
);


	%END;

%mend FINALIZEATTRITION;
%FINALIZEATTRITION();

proc sort data = _waterfall;
by Group CritNum;
run;

data MSOC.&REQUESTID.&RUNID._AttritionTable;
set _Waterfall;
by group;

Excluded = lag(CritCnt) - CritCnt;
if first.group then Excluded = .;

rename CritCnt = Remaining;

run;

/* MODIF WAVE 3 BLOCK START */
proc sort data=msoc.&REQUESTID.&RUNID._stockpiling_excl out=_temp;
by Group;
run;

data _temp;
retain Group CritNum CritDesc Remaining ExclMembers;
set _temp(keep=Group ClaimType ExclMembers);
format CritDesc $200.;
rename ExclMembers=Excluded;
Remaining=.;
if ClaimType="QUERY" then do;
	CritDesc="Information - Members with at least one QUERYFILE claim with supply and/or amount outside specified ranges";
	CritNum=18;
end;
if ClaimType="EVENT" then do;
	CritDesc="Information - Members with at least one EVENTFILE claim with supply and/or amount outside specified ranges";
CritNum=19;
end;
if ClaimType="CONDITION" then do;
	CritDesc="Information - Members with at least one CONDFILE claim with supply and/or amount outside specified ranges";
CritNum=20;
end;
drop ClaimType;
run;

data MSOC.&REQUESTID.&RUNID._AttritionTable;
set MSOC.&REQUESTID.&RUNID._AttritionTable _temp;
run;

proc sort data=MSOC.&REQUESTID.&RUNID._AttritionTable;
by Group CritNum;
run;

/* MODIF WAVE 3 BLOCK END */

/*MODIF WAVE3*/
data _premaster1
     _premaster2
	 _premaster3;
   set _premaster;

   if HadIncEvent = 0 and HadCond = 1 and ExclCritMeet = 1;

   if EpWashTyp = "SING" then output _premaster1;
   else if WashTyp="SING" then output _premaster2; 
   else output _premaster3;

drop IncEventDum ExclEligDum ExcCondDum IncCondDum;
run;

proc sort nodupkey data = _premaster1;
   by PatId Group;
run;   

proc sql noprint;
create table _FirstEpisEvent as 
select Group, PatId, Episode, min(EventDt) as FEventDt from _premaster2
group by Group, PatId
order by Group, PatId, Episode;
quit;

proc sort data = _premaster2;
by Group PatId Episode;
run;

data _premaster2;
merge _premaster2 _FirstEpisEvent;
by Group PatId Episode;
if not missing(FEventDt) and EpisodeStartDt>FEventDt then delete;
drop FEventDt;
run;

data DPLocal.&REQUESTID.&RUNID._MasterTable;
   set _premaster1
       _premaster2
	   _premaster3;
run;

proc datasets library=work nolist nowarn;
   delete _premaster1 _premaster2 _premaster3;
quit;

/*---------------------------------------------------------------------------*/
/*  12 -- Process claims for denominators; determine index dates             */
/*---------------------------------------------------------------------------*/

%MACRO DENOMLOOP();

data _null_;
   set _GroupList end=fin;
   if fin then call symput('NQUERYGROUP',_n_);
run;

   %PUT &NQUERYGROUP.;
   %DO I = 1 %TO &NQUERYGROUP.;

data _null_;
   set _GroupList;
   if _n_ = &i. then call symput('ITGROUP',strip(Group));
run;

      %PUT &ITGROUP.;   

/*Select this Query class specific washout period*/
data _null_;
   set _querysettings;
   if Group in("&ITGROUP.") then do;
      call symput('GROUPWASHOUT',WashPer);
      call symput('GROUPWASHTYP',strip(WashTyp));
      call symput('GROUPMINEPISDUR',Max(MinEpisDur,1));
	  call symput('GROUPENRDAYS',EnrDays);
   end;
run;

      %PUT &GROUPWASHOUT. &GROUPWASHTYP. &GROUPMINEPISDUR. &GROUPENRDAYS.;

/*Select this Query class specific event settings*/
      %LET EVENTWASHOUT = 0;
      %LET EVENTWASHTYP = MULT;

data _null_;
   set _eventsettings;
   if Group in("&ITGROUP.") then do;
      call symput('EVENTWASHOUT',WashPer);
      call symput('EVENTWASHTYP',strip(WashTyp));            
   end;   
run;

      %PUT &EVENTWASHOUT. &EVENTWASHTYP.;

/*filter according to pre-existing conditions*/
      %LET CONDLOOKTYP=;

data _null_;
   set _condfile;
   if Group in("&ITGROUP.") then do;
      call symput('CONDLOOKTYP',strip(CondLookTyp));
   end;
run;

      %PUT &CONDLOOKTYP.;
         
      %LET INCLREQD=0;  /*No INCLUSION for this QUERYGROUP*/
      %LET EXCLREQD=0;  /*No EXCLUSION for this QUERYGROUP*/

data _LoopCondFile;
   set _condfile;

   if Group = "&ITGROUP.";
      
   if Inclusion=1 then call symput('INCLREQD',1);
   if Inclusion=0 then call symput('EXCLREQD',1);
run;

      %PUT &INCLREQD. &EXCLREQD.;   
         
      %MACRO WRAPPER();
      /*If pre-existing conditions are required need to select members with pre-existing conditions*/
         %IF %EVAL(&INCLREQD.>0 or &EXCLREQD.>0) %THEN %DO;
            %IF &INCLREQD. = 1 %THEN %DO;

proc sort nodupkey data = _masternonqueryfile
                    out = _cond(keep=PatId ADate CondFrom CondTo);
   by PatId ADate CondFrom CondTo;
   where cond and Group in("&ITGROUP.") and Inclusion = 1;
run;

/*Must re-define enrollment episodes to scan only when members meeting the pre-existing
                   condition requirement*/
proc sql noprint;
Create table _denomint2 as
Select enrol.*,
       claims.ADate,
       claims.CondFrom,
       claims.CondTo,
       max(claims.Adate - claims.CondTo,enrol.Enr_Start) as PDate format mmddyy10.,
       min(claims.Adate - claims.CondFrom,enrol.Enr_End) as EDate format mmddyy10.   
   from _Denomint as enrol,
        _cond as claims
   where enrol.PatId = claims.PatId and enrol.Enr_Start <= claims.Adate - claims.CondFrom and 
                                        enrol.Enr_End >= claims.Adate - claims.CondTo
   order PatId, EligEPisode, PDate; 
quit;

/*for a same eligible record, identify overlapping PDate - EDate intervals*/
data _MDenomint;
   set _denomint2;
   by PatId EligEpisode;
   format lEDate mmddyy10.;

   lEDate = lag(EDate);

   diff=PDate-lEDate-1;

   if first.EligEpisode then do;
      lPDate=.;
      diff=.;
      PEpisode=1;
   end;
   else do;
      if diff > 0 then do;
         PEpisode = PEpisode + 1;
      end;
   end;

   retain Pepisode;
run;

proc means data=_MDenomint nway noprint;
   var PDate EDate;
   class Patid EligEpisode PEpisode;
   id Enr_Start Enr_end Birth_Date MinAgeDate MaxAgeDate sex;
   output out=_MDenomint(drop=_:) min(PDate)=  max(EDate)=;
run;

            %END;
            %ELSE %DO;

data _MDenomInt;
   set _DenomInt(where=(enr_end >= &QUERYFROM. and enr_start <= &QUERYTO.));
   format PDate EDate mmddyy10.;
   PDate = max(Enr_Start,&QUERYFROM.);
   EDate = min(Enr_End,&QUERYTO.);
   PEpisode = EligEpisode;
   keep PatId PDate EDate Enr_Start Enr_end Birth_Date MinAgeDate MaxAgeDate sex EligEpisode PEpisode;
run;

            %END;
            %IF &EXCLREQD. = 1 %THEN %DO;

proc sort nodupkey data=_LoopCondFile;
   by CondFrom CondTo;
   where inclusion=0;
run;

data _LoopCondFile;
   set _LoopCondFile end=fin;
   where inclusion=0;
   ExclNum=_N_;
   if fin then call symput("NumExcl",ExclNum);
run;

               %PUT &NumExcl.;

proc sql noprint;
create table _MDenomint_ as
Select pdates.*,
       lookup.CondFrom,
       lookup.CondTo,
       lookup.ExclNum 
   from _LoopCondFile(keep=CondFrom CondTo ExclNum) as lookup,
        _MDenomint as pdates
   order by PatId, EligEpisode, PEpisode, ExclNum;
quit;

data _MDenomint;
   set _MDenomint_;
   format MaxPDate MinEDate mmddyy10.;

   if CondFrom < 0 then MaxPdate = Enr_Start - CondFrom;
   else MaxPDate = PDate;

   if CondTo > 0 then MinEDate = Enr_End - CondTo;
   else MinEDate = EDate;

               /*check if above remain consistent*/

   PDate = max(PDate,MaxPdate);
   EDate = min(EDate,MinEDate);

run;

proc means data=_MDenomint nway noprint;
   var PDate EDate;
   class Patid EligEpisode PEpisode;
   id Enr_Start Enr_end Birth_Date MinAgeDate MaxAgeDate sex;
   output out=_MDenomint(drop=_: where=(PDate<=EDate)) min(PDate)=  max(EDate)=;
run;

               /*go get exclusion claims*/
proc sort nodupkey data = _masternonqueryfile
                    out = _cond(keep=PatId ADate CondFrom CondTo);
   by PatId ADate CondFrom CondTo;
   where cond and Group in("&ITGROUP.") and Inclusion = 0;
run;

data _cond;
   set _cond;
   format ExcPdate ExcEdate mmddyy10.;

   ExcPdate = Adate - CondTo;
   ExcEdate = Adate - CondFrom;

run;

proc sort data = _cond;
   by PatId ExcPdate;
run;

data _cond;
   set _cond;
   by Patid;
             
   lExcEdate = lag(ExcEdate);
   diff = ExcPDate - lExcEdate - 1;

   if first.PatId then do;
      lExcEdate = .;
      diff = .;
      ExcEpisode = 1;
   end;
   else do;
      if diff > 0 then ExcEpisode = ExcEpisode + 1;
   end;

   retain ExcEpisode;
run;

proc means noprint data = _cond nway;
   var ExcPdate ExcEdate;
   class PatId ExcEpisode;
   output out=_cond(drop=_: ExcEpisode) min(ExcPdate)= max(ExcEdate)=;
run;

               /*left join here with _MDenomint (watch out for multiple merge)*/
proc sql noprint;
create table _MDenomint_ as
Select pdates.*,
       exdates.ExcPdate,
       exdates.ExcEdate
   from _MDenomint as pdates left join
        _cond as exdates
      on pdates.PatId = exdates.PatId;
quit;

data _MDenomint(where=(PDate<=EDate));
   set _MDenomint_;

   if (ExcPDate <= PDate and  PDate <= ExcEDate <= EDate) then do;
      PDate = ExcEDate + 1;
      EDate = EDate;
      output;
   end;
   else if ExcPDate > PDate and ExcEDate < EDate then do; 
      PDate = PDate;
      EDate = ExcPDate - 1;
      output;

      PDate = ExcEDate + 1;
      EDate = EDate;
      output;
   end;
   else if ExcPDate <= EDate and ExcEDate >= EDate then do;
      PDate = PDate;
      EDate = ExcPDate-1;
      output;
   end;
   else output; 
run;

proc sort data = _MDenomint;
   by PatId PDate;
run;

data _MDenomint;
   set _MDenomint;
   by PatId;

   lEDate = lag(EDate);
   diff = PDate - lEDate- 1;

   if first.PatID then do;
      lEDate = .;
      diff = .;
      PEpisode = 1;
   end;
   else do;
      if diff > 0  then PEpisode = PEpisode + 1;
   end;

   retain PEpisode;
run;

proc means noprint data = _MDenomint nway;
   var PDate EDate;
   class PatId EligEpisode PEpisode;
   id Enr_Start Enr_end Birth_Date MinAgeDate MaxAgeDate sex;
   output out=_MDenomint(drop=_:) min(PDate)= max(EDate)=;
run;

            %END;

data _MDenomInt;
   set _MDenomInt(where=(enr_end >= &QUERYFROM. and enr_start <= &QUERYTO.));

   Enr_End = min(Enr_End,EDate);
            
   if enr_end < MinAgeDate then delete;

   LastAgeGroup=&NUMAGECAT.;
   do i=&NUMAGECAT. to 1 by -1;
      Threshdate=intnx(scan("&AGETYP.",i),birth_date,scan("&AGETHRESH.",i),'sameday'); 
      if Threshdate<= min(&QUERYTO.,Enr_end) then do;
         LastAgeGroup =i;
         FirstAgeGroup=i;
         if Threshdate > max(&QUERYFROM.,Enr_Start)  then do;
            do j=i-1 to 1 by -1;
               if intnx(scan("&AGETYP.",j),birth_date,scan("&AGETHRESH.",j),'sameday') <= 
                             max(&QUERYFROM.,Enr_Start) then do;
                  FirstAgeGroup=j;
                  leave;
               end;
            end;
         end;
         leave;
      end; 
   end;
   drop i j threshdate;
run;

         %END; /*end for either inclusion or exclusion*/
         %ELSE %DO;

data _MDenomInt;
   set _DenomInt(where=(enr_end >= &QUERYFROM. and enr_start <= &QUERYTO.));
   format PDate mmddyy10.;
   PDate = .;
   PEpisode=1;
run;

         %END;
      %MEND WRAPPER;
      %WRAPPER();

/*Creating Claims Dates for events*/
proc sort nodupkey data= _masternonqueryfile 
                    out= _EClaims(keep=PatId ADate Group);
   by PatId ADate; 
   where (Event or IncEvent) and Group in("&ITGROUP.");
run;
         
data _EClaims;
   set _EClaims;
   format ExpireDt mmddyy10.;
   Expiredt=Adate;
run;

proc sort data = _EClaims;
   by PatId ADate Expiredt;
run;

proc sort data= _masterqueryfile 
           out= _DClaims(keep=PatId ADate Expiredt Group);
   by PatId ADate Expiredt; 
   where Group in("&ITGROUP.") or IGroup in("&ITGROUP.","_ALLGROUPS_");
run;
      
/*Flag if query or event claim since washout regime can be different*/
data _Claims3;
   set _EClaims(in=a) _DClaims(in=b);
   by PatId ADate ExpireDt; 

   QueryClaim = 0;
   EventClaim = 0;
   if a then EventClaim = 1;
   if b then QueryClaim = 1;

run;

proc sql noprint;
create table _Claims as 
select PatId,
       ADate,
       Group,
       ExpireDt,
       max(QueryClaim) as QueryClaim,
       max(EventClaim) as EventClaim
   from _Claims3
   group by PatId, ExpireDt;
quit;

/*list for pre-cleaning*/
proc sort nodupkey data=_Claims out=_ClaimList(keep=patid);
   by PatId;
run;

/*Creating Dates*/      
data _AllMember;
   Merge _MDenomInt(in=a) _ClaimList(in=b);  
   by PatId;
   if a;
   withclaims = 0;
   if a and b then withclaims=1;
   format QueryStartDate mmddyy10.;
   QueryStartDate=Max(&QUERYFROM.,MinAgeDate,enr_start,PDate);  /*Minimum date the patient can have an HOI*/
   Enr_Start=Max(QueryStartDate-max(&GROUPWASHOUT.,&EVENTWASHOUT.,&GROUPENRDAYS.),enr_start);
   enr_end=min(&QUERYTO.,enr_end,MaxAgeDate);
   if enr_end >= enr_start; 

   keep PatId EligEpisode PEpisode Birth_Date Sex MinAgeDate MaxAgeDate Enr_Start Enr_End 
        LastAgeGroup FirstAgeGroup PDate withclaims QueryStartDate;
run;
      
Proc SQL Noprint;
Create Table _Claims2 as
Select Enrol.*,
       claims.ADate,
       claims.Expiredt,
       claims.Group,
       claims.QueryClaim,
       claims.EventClaim
   From _AllMember(where=(withclaims=1)) as Enrol,
        _Claims as claims
   Where Enrol.PatId=claims.PatId and Enrol.enr_Start <= claims.ADate <= Enrol.enr_end 
   order Patid, EligEpisode, PEpisode, Adate; /*remove claims outside of period BEFORE AND AFTER*/ 
quit;

/*To treat overlapping claims and create effective washout*/
data _claims2 
     _claimmark(keep=PatId EligEpisode PEpisode QueryClaim EventClaim ClmNum lEffWashDt rename=(lEffWashDt=EffWashDt));
   set _claims2;
   by PatId EligEpisode PEpisode;
   format EffWashDt lEffWashDt mmddyy10.;


   EffWashDt = ExpireDt + max(&GROUPWASHOUT.*QueryClaim,&EVENTWASHOUT.*EventClaim);

   if first.PEpisode then do;
      ClmEpisode = 1;
      ClmNum = 0;
      lExpireDt = .;
      lEffWashDt = .;
   end;

   if lExpireDt ne . then do;
      diff = ADate - lExpiredt - 1;
      ClmEpisode = ClmEpisode + 1;
   end;
            
   ClmNum = ClmNum + 1;

   lexpiredt = max(expiredt,lexpiredt);

   lEffWashDt = max(EffWashDt,lEffWashDt);

   retain lexpiredt ClmEpisode ClmNum lEffWashDt;
run;

proc means noprint data = _claims2 nway;
   var ADate Expiredt ClmNum;
   class PatID EligEpisode PEpisode ClmEpisode;
   id Birth_Date MaxAgeDate MinAgeDate Sex LastAgeGroup FirstAgeGroup QueryStartDate Group Enr_Start;
   output out=_claims2(drop=_:) min(ADate)= 
                                max(ExpireDt)= 
                                max(ClmNum)=;
run;

proc sql noprint;
Create Table _Claims3 as
Select clmep.PatId, 
       clmep.EligEpisode,
       clmep.PEpisode,
       clmep.ADate, 
       clmep.ExpireDt,
       clmep.Sex, 
       clmep.Birth_date,
       clmep.MinAgeDate, 
       clmep.MaxAgeDate,
       clmep.QueryStartDate, 
       clmep.FirstAgeGroup,
       clmep.LastAgeGroup, 
       clmep.Group,
	   clmep.Enr_Start,
       clmdu.EffWashDt,
       clmdu.EventClaim, 
       clmdu.QueryClaim
   From _claims2 as clmep,
        _claimmark as clmdu
   Where clmep.PatId = clmdu.PatId and clmep.EligEpisode = clmdu.EligEpisode and 
         clmep.PEpisode = clmdu.PEpisode and clmep.ClmNum = clmdu.ClmNum
   order by PatId,EligEpisode,PEpisode,ADate;
quit;

/*Create begin and end intervals (what if more than 1 enrol; episode,
      will have duplicate claim when incident respective to own, should be OK)*/
data _DenomToLoop;
   set  _AllMember(in=a keep=Patid EligEpisode Pepisode Enr_Start Sex birth_date MinAgeDate MaxAgeDate QueryStartDate 
                             FirstAgeGroup LastAgeGroup rename=Enr_Start=Adate)
        _Claims3(keep=Patid EligEpisode Pepisode ADate Expiredt Sex birth_date MinAgeDate MaxAgeDate QueryStartDate 
                      FirstAgeGroup LastAgeGroup Group EffWashDt EventClaim QueryClaim Enr_Start)
        _AllMember(in=b keep=Patid EligEpisode Pepisode  Enr_End Sex birth_date MinAgeDate MaxAgeDate QueryStartDate 
                             FirstAgeGroup LastAgeGroup Enr_Start rename=Enr_End=Adate);
   by Patid EligEpisode Pepisode ADate;

   if a then do;
      Group="BEGINELIG";
      Expiredt=Adate;
      EventClaim = 1;
      QueryClaim = 1;
      EffWashDt = ExpireDt + max(&GROUPWASHOUT.*QueryClaim,&EVENTWASHOUT.*EventClaim);
	  Enr_Start=ADate;
   end;
   if b then do;
      Group="ENDELIG";
      Expiredt=Adate;
      EventClaim = 0;
      QueryClaim = 0;
      EffWashDt = ExpireDt + max(&GROUPWASHOUT.*QueryClaim,&EVENTWASHOUT.*EventClaim);
   end;

   IsClaim=0;
   if not a and not b then IsClaim=1;

run;


data _DenomToLoop(rename=(lEffWashDt=EffWashDt));
   set _DenomToLoop;
   by PatId EligEpisode Pepisode;
   format lEffWashDt mmddyy10.;

   if first.PEpisode then do;
      lEffWashDt = .;            
   end;

   lEffWashDt = max(EffWashDt,lEffWashDt);

   retain lEffWashDt;
   drop EffWashDt;
run;

/*Creating HOI Free episodes from gap-less Enrollment sequences*/
data _FindEpisodes(rename=(lEventClaim=EventClaim
                           lQueryClaim=QueryClaim
                           lEffWash=EffWash));
   set _DenomToLoop;
   by PatId EligEpisode Pepisode;

   EffWash = EffWashDt - ExpireDt - 1*IsClaim;

   lGroup = lag(Group);
   lExpireDt = lag(ExpireDt);
   lEffWash = lag(EffWash);
   lEventClaim = lag(EventClaim);
   lQueryClaim = lag(QueryClaim);

   if first.Pepisode then do;
      lExpireDt = .;
      lGroup = "";
      lEffWash = .;
      lEventClaim = .;
      lQueryClaim =.;
   end;

   if lExpiredt ne . then diff=ADate-lExpireDt;

   format lExpireDt StartDate EndDate mmddyy10.;

   StartDate = lExpireDt + 1 - (lGroup eq "BEGINELIG");
   EndDate = ADate;
   QueryStartDate=Max(QueryStartDate,StartDate);

   if MinAgeDate <= EndDate and MaxAgeDate >= StartDate;

   if diff >= 0 and QueryStartDate <= EndDate;

   if EndDate >= &QUERYFROM. and StartDate <= &QUERYTO.;
 
   FirstPer=intck("Months",&QUERYFROM.,QueryStartDate)+1;
   LastPer=intck("Months",&QUERYFROM.,EndDate)+1;

   keep PatId Birth_Date sex StartDate QueryStartDate EndDate Enr_Start first: last: lEventClaim lQueryClaim lEffWash;  
run;

/*Get index dates:
         Six possibilities
            1 - Mult washout for event and drug, no need to worry about index dates or event dates 
               (output index date table for code execution)

            2 - Mult washout for event and Min washout for drug, can only have one drug episode, 
                need to retrieve index date and see if any claim before the index date

            3 - Min washout for event and mult washout for drug, can have more than one drug episode until event, 
                get first event or inc event date in period and see if we have also before study

            4 - Min washout for event and min washout for drug, can have only one drug episode

            5 - Min washout for event and sing washout for drug, eligible to have an episode until event claim,
                single episode

            6 - Mult washout for event and sing washout for drug, single episode
*/

      %MACRO WRAPPER();
         %IF %STR("&EVENTWASHTYP.")=%STR("MULT") and %STR("&GROUPWASHTYP.")=%STR("MULT") %THEN %DO;

data _TermDtTable;
   set Dplocal.&REQUESTID.&RUNID._MasterTable(where=(Group in("&ITGROUP.")));
   format TermDt mmddyy10.;
   TermDt=EpisodeStartDt;
   keep PatId TermDt;
run;

/*keeping TermDt, only useful for minimum washout cases when each PatId will have a single index date*/
proc sort nodupkey data = _TermDtTable;
   by PatId;  
run;

         %END;
         %ELSE %IF %STR("&EVENTWASHTYP.")=%STR("MULT") and %STR("&GROUPWASHTYP.")=%STR("MIN") %THEN %DO;

proc means noprint data = _DClaims nway;
   var ADate;
   Class PatId;
   output out=_TermDtTable(drop=_:) min=TermDt;
run;

         %END;
         %ELSE %IF %STR("&EVENTWASHTYP.")=%STR("MIN") and %STR("&GROUPWASHTYP.")=%STR("MULT") %THEN %DO;

proc means noprint data = _EClaims nway;
   var ADate;
   Class PatId;
   output out=_TermDtTable(drop=_:) min=TermDt;                   
run;

         %END;
         %ELSE %IF %STR("&EVENTWASHTYP.")=%STR("MIN") and %STR("&GROUPWASHTYP.")=%STR("MIN") %THEN %DO;

proc means noprint data = _DClaims nway;
   var ADate;
   Class PatId;
   output out=_DTermDtTable(drop=_:) min=DTermDt;
run;

proc means noprint data = _EClaims nway;
   var ADate;
   Class PatId;
   output out=_ETermDtTable(drop=_:) min=ETermDt;                   
run;

data _TermDtTable;
   merge _DTermDtTable _ETermDtTable;
   by PatId;
   format TermDt mmddyy10.;
   TermDt = min(DTermDt,ETermDt);
   keep PatId TermDt;
run;

         %END;
         %ELSE %IF %STR("&EVENTWASHTYP.")=%STR("MIN") and %STR("&GROUPWASHTYP.")=%STR("SING") %THEN %DO;

data _DTermDtTable;
   set Dplocal.&REQUESTID.&RUNID._MasterTable(where=(Group in("&ITGROUP.")));
   format DTermDt mmddyy10.;
   DTermDt=EpisodeStartDt;
   keep PatId DTermDt;
run;

proc sort data = _DTermDtTable;
   by PatId;
run;

proc means noprint data = _EClaims nway;
   var ADate;
   Class PatId;
   output out=_ETermDtTable(drop=_:) min=ETermDt;                   
run;

data _TermDtTable;
   merge _DTermDtTable _ETermDtTable;
   by PatId;
   format TermDt mmddyy10.;
   TermDt = min(DTermDt,ETermDt);
   keep PatId TermDt;
run;

         %END;
         %ELSE %IF %STR("&EVENTWASHTYP.")=%STR("MULT") and %STR("&GROUPWASHTYP.")=%STR("SING") %THEN %DO;

data _TermDtTable;
   set Dplocal.&REQUESTID.&RUNID._MasterTable(where=(Group in("&ITGROUP.")));
   format TermDt mmddyy10.;
   TermDt=EpisodeStartDt;
   keep PatId TermDt;
run;

proc sort data = _TermDtTable;
   by PatId;
run;

         %END;		 
		 %ELSE %IF %STR("&EVENTWASHTYP.")=%STR("SING") and %STR("&GROUPWASHTYP.")=%STR("MIN") %THEN %DO;
/*MODIF WAVE3*/
* COHORTDEF=03, Query washout is missing and Event Wahout is non missing;
proc means noprint data = _DClaims nway;
   var ADate;
   Class PatId;
   output out=_TermDtTable(drop=_:) min=TermDt;
run;

         %END;		 
		 %ELSE %IF %STR("&EVENTWASHTYP.")=%STR("SING") and %STR("&GROUPWASHTYP.")=%STR("MULT") %THEN %DO;
* COHORTDEF=03 and Both Query and Event Washout are not missing;
data _TermDtTable;
   set Dplocal.&REQUESTID.&RUNID._MasterTable(where=(Group in("&ITGROUP.") and not missing(EventDt)));
   format TermDt mmddyy10.;
   TermDt=EventDt;
   keep PatId TermDt;
run;

proc sort data = _TermDtTable;
   by PatId;
run;

         %END;
      %MEND WRAPPER;
      %WRAPPER();

data _Patient_days_M _Patient_days_F _Patient_days_U;
   merge _FindEpisodes(in=a) _TermDtTable;
   by Patid;
   if a;

   if sex='M' then output _Patient_days_M;
   else if sex='F' then output _Patient_days_F;
   else if sex='U' then output _Patient_days_U;

run;

      %LET sex=M;
   
      %MACRO ACCUMDENOM(sex=);

/*Here we will accumulate patients and patient-days to obtain up to 3 records at the end
                    which will constitute the denominators for the modular program*/
data _Patient_days_&sex.;
   set _Patient_days_&sex. end=eof;
   by Patid;

/*Naming convention of arrays:
      prefix _ means temporary vector
      prefix Inc = incident
      
      Suffix AG  = Age group stratification
      Suffix Per = period stratification (e.g., weekly, monthly, quarterly) 
      Suffix Y = year stratification

      When "Days" is not included in name = we`re counting "members"
      When "Days" is included in name = we`re counting "members-days"
*/

/*Age Groups*/
   array _IncAg(*) _IncAg1-_IncAg&NUMAGECAT. ;      

   array IncAg(*) IncAg1-IncAg&NUMAGECAT.;      
   array IncDaysAg(*) IncDaysAg1-IncDaysAg&NUMAGECAT.;      

/*Periodicity*/      
   array _IncPer(*) _IncPer1-_IncPer&NUMPER. ;      

   array IncPer(*) IncPer1-IncPer&NUMPER.;      
   array IncDaysPer(*) IncDaysPer1-IncDaysPer&NUMPER.;      

/*Year*/
   array _IncY(*) _IncY&FROMY.-_IncY&TOY. ;

   array IncY(*) IncY&FROMY.-IncY&TOY.;
   array IncDaysY(*) IncDaysY&FROMY.-IncDaysY&TOY.;

   if first.patid then do;
      call missing(of _Inc:);
   end;

/*Can put 0 for minimum episode duration, must floor at 1 since 0 days episode duration 
        is not possible*/
   DaysInExcess = EndDate - Max(StartDate+EffWash,QueryStartDate) + 1;


   do i=FirstAgeGroup to LastAgeGroup;
      format StartAgeDate EndAgeDate mmddyy10. ;

      /*Compute Strata Start/End date*/
      StartAgeDate=intnx(scan("&AGETYP.",i),birth_date,scan("&AGETHRESH.",i),'sameday');
      if i<&NUMAGECAT. then 
                    EndAgeDate=intnx(scan("&AGETYP.",i+1),birth_date,scan("&AGETHRESH.",i+1),'sameday') - 1;
      else if &MAXAGE.=99999 then EndAgeDate=intnx("years",birth_date,110,'sameday');
      else EndAgeDate=intnx(scan("&AGETYP.",&NUMAGECAT.),birth_date,&MAXAGE.+1,'sameday')-1;
               
      if EndDate<StartAgeDate then leave;  /*this age category outside this continuous eligibility*/

      if QueryStartDate<=EndAgeDate then do;                  

         /*Incidence*/
         NumWashOutDays=min(EndAgeDate,EndDate)-StartDate;
		 NumEnrDays=min(EndAgeDate,EndDate)-Enr_Start;
         NumDays=min(min(EndAgeDate,EndDate)-max(QueryStartDate,StartAgeDate)+1,NumWashOutDays-EffWash+1,NumEnrDays-&GROUPENRDAYS.+1);                     

         if DaysInExcess >= &GROUPMINEPISDUR. and (NumWashOutDays-EffWash) >= 0 and NumEnrDays >= &GROUPENRDAYS. then do;
            if %STR("&EVENTWASHTYP.")=%STR("MULT") and %STR("&GROUPWASHTYP.")=%STR("MULT") then do;
               _IncAg(i)=1;
               IncDaysag(i)=sum(NumDays,IncDaysAg(i));
            end;
            else if TermDt eq . or (TermDt ne . and StartDate<=TermDt) then do;  /*WashTyp='Min' or 'Sing'*/
               _IncAg(i)=1;
               IncDaysag(i)=sum(NumDays,IncDaysAg(i));
            end;
         end;
      end;            
   end;


   do i=FirstPer to LastPer;
      format StartPerDate EndPerDate mmddyy10.;

/*Here the use of the modulo operator has been chosen to avoid having to call the intnx() function to
       track year change as it was too much time consuming when this was tested in a synthetic real-life 
       size problem*/

      /*Year*/
      if mod(i,12)=&YEARCHANGE. or  i=FirstPer then do;
         year=int(&FROMY.+(i+&FORMOD.)/12);
         j=year-&FROMY.+1;

         /*Incidence*/
         NumWashOutDays=min(mdy(12,31,year),EndDate)-StartDate;
		 NumEnrDays=min(mdy(12,31,year),EndDate)-Enr_Start;
         /*number of query days overlapping this year*/
         NumDays=min(min(mdy(12,31,year),EndDate)-max(QueryStartDate,mdy(1,1,year))+1,NumWashOutDays-EffWash+1,NumEnrDays-&GROUPENRDAYS.+1);                     

         if DaysInExcess >= &GROUPMINEPISDUR. and (NumWashOutDays-EffWash) >= 0 and NumEnrDays >= &GROUPENRDAYS. then do;
            if %STR("&EVENTWASHTYP.")=%STR("MULT") and %STR("&GROUPWASHTYP.")=%STR("MULT") then do;
               _IncY(j)=1;
               IncDaysY(j)=sum(NumDays,IncDaysY(j));
            end;
            else if TermDt eq . or (TermDt ne . and StartDate<=TermDt) then do;  /*WashTyp='Min' or 'Sing'*/
               _IncY(j)=1;
               IncDaysY(j)=sum(NumDays,IncDaysY(j));
            end;
         end;
      end;

      /*Year/Mont*/
      /* Compute Strata Start/End date*/
      if i=FirstPer then StartPerdate=mdy(mod(i+&FORMOD.,12)+1,1,int(&FROMY.+(i+&FORMOD.)/12));
      else StartPerdate=EndPerDate+1;
      EndPerdate=mdy(mod(i+&FORMOD.+1,12)+1,1,int(&FROMY.+(i+&FORMOD.+1)/12))-1;

      if EndDate<StartPerDate then leave;

      /*Is there an overlap with this strata?*/
      if QueryStartDate<=EndPerDate then do;

         /*Incidence*/
         NumWashOutDays=min(EndPerDate,EndDate)-StartDate;
		 NumEnrDays=min(EndPerDate,EndDate)-Enr_Start;
         NumDays=min(min(EndPerDate,EndDate)-max(QueryStartDate,StartPerDate)+1,NumWashOutDays-EffWash+1,NumEnrDays-&GROUPENRDAYS.+1);                     

         if DaysInExcess >= &GROUPMINEPISDUR. and (NumWashOutDays-EffWash) >= 0 and NumEnrDays >= &GROUPENRDAYS. then do;
            if %STR("&EVENTWASHTYP.")=%STR("MULT") and %STR("&GROUPWASHTYP.")=%STR("MULT") then do;
               _IncPer(i)=1;
               IncDaysPer(i)=sum(NumDays,IncDaysPer(i));
            end;
            else if TermDt eq . or (TermDt ne . and StartDate<=TermDt) then do;  /*WashTyp='Min' or 'Sing'*/
               _IncPer(i)=1;
               IncDaysPer(i)=sum(NumDays,IncDaysPer(i));
            end;
         end;
      end;
   end;

   if last.patid then do;  /*With this stage, all Y are filled and accumulated*/

      IncALL=sum(IncALL,max(of _IncY:,.));      /*only one presence is enough*/

      do i=FirstAgeGroup to LastAgeGroup;               
         IncAg(i)=sum(IncAg(i),_IncAg(i));
      end;                
      do i = 1 to &NUMYEARS.;               
         IncY(i)=sum(IncY(i),_IncY(i));
      end;
      do i = 1 to &NUMPER.;
         IncPer(i)=sum(IncPer(i),_IncPer(i));
      end; 
   end;

   if eof then output;            

   retain _Inc: Inc:;

   keep Patid birth_date Sex StartDate EndDate QueryStartDate Inc:;
               
run;
         %MEND ACCUMDENOM;
         %ACCUMDENOM(sex=M);
         %ACCUMDENOM(sex=F);
         %ACCUMDENOM(sex=U);

%MACRO SQUARE(file=,Sex=);
	proc contents data=&file. noprint out=_tilt_ ;
	data _null_ ;
	set _tilt_ ;
	call symput('nobs',trim(left(put(nobs,15.)))) ;
	run;

   	%PUT &pnobs.;

	%IF %EVAL(&nobs.=0) %THEN %DO;
		proc sql noprint;
			insert into &file (Sex) values("&sex.");
		quit;
	%END;
%MEND SQUARE;
%SQUARE(file=_Patient_days_F,sex=F);
%SQUARE(file=_Patient_days_M,sex=M);
%SQUARE(file=_Patient_days_U,sex=U);

data  _MasterDenomTable;
   set _Patient_days_F(in=a)
       _Patient_days_M(in=b)
       _Patient_days_U(in=c);
            
   IncDaysALL=sum(of IncDaysY:,.);       

   keep sex inc:;
run;

proc transpose data = _MasterDenomTable
                out = _MasterDenomTable(rename=(_NAME_ = Segment COL1=count));
   by sex;
run;

proc sort data = _MasterDenomTable;
   by segment sex;
run;

data _MasterDenomTable;
   format Group $30. Segment $16. count best12.;
   set  _MasterDenomTable;
   Group = "&ITGROUP.";
   label Segment='Segment';
   if missing(count) then count=0;
run;

      %IF &I.=1 %THEN %DO;

data DPLocal.&REQUESTID.&RUNID._MasterDenomTable;
   set _MasterDenomTable;
run;       

      %END;
      %ELSE %DO;

proc append base = DPLocal.&REQUESTID.&RUNID._MasterDenomTable
            data = _MasterDenomTable force;
run;      

      %END;       
   %END;
%MEND DENOMLOOP;
%DENOMLOOP();

/*---------------------------------------------------------------------------*/
/*  13 -- Post processing denominator results                                */
/*---------------------------------------------------------------------------*/

data _denom;
   set  DPLocal.&REQUESTID.&RUNID._MasterDenomTable;

   if upcase(segment)=:'INC' then do;
      segment=upcase(substr(segment,4,12));
      Inc=1;
   end;

   time=0;
   if upcase(segment)=:'DAYS' then   do;
      segment=upcase(substr(segment,5,11));
      time=1;
   end;

   /*Assign Age Groupings for merging and create year month variables*/
   if upcase(Segment)=:"AG" then do;
      i=compress(upcase(Segment),"AG");
      format AgeGroup $13.;
      AgeGroup = scan("&AGESTRAT.",i,' ');
   end;
   if upcase(Segment)=:"PER" then do;
      StrataStart=intnx("Months",&QUERYFROM.,compress(upcase(Segment),"PER")-1,'sameday');
      Year=year(StrataStart);
      Month=month(StrataStart);
   end;
   if  upcase(Segment)=:"Y" then Year=compress(upcase(Segment),"Y");
   
   keep segment inc Group count sex Year month AgeGroup time;
run;
   
proc sort data=_denom;
   by Group segment sex inc time count;
run;

data _denom;
   format DPID SITEID $2.;
   merge _denom(where=(inc=1 and time=0) rename=count=IncDenCount)         
         _denom(where=(inc=1 and time=1) rename=count=IncDaysCount);
   by Group segment sex;
   DPID="&DPID.";                                    
   SITEID="&SITEID."; 
   drop inc /*Segment time*/;
run;   

proc sql noprint;
create table MSOC.&REQUESTID.&RUNID._DenTable0 as
Select DPID,
       SITEID,
       Group,
       Sex,
       AgeGroup,
       year,
       month,
       IncDenCount,
       IncDaysCount
   from _denom;
quit;

/***********************************************************/
/* Add demographics information to master dispensing table */
/***********************************************************/

proc sort nodupkey data = _Enrollment(keep=PatId Birth_Date Sex);
   by PatId;
run;

data DPLocal.&REQUESTID.&RUNID._QueryGroup(keep=PatID Sex AgeGroup Group EpisodeRxSup /*MODIF WAVE3*/ EpisodeRxAmt NumDispensing 
                                                        Year Month tte patient HadEvent /*MODIF WAVE3*/ AllEvents
                                                        PCHRLSON CHRLSON XCHRLSON NumVisit
                                                        BMIz_Strat                                                        
                                                  );
   if 0 then set _Enrollment;
   declare hash ht (hashexp:16, dataset:"_Enrollment");
   ht.definekey('PatId');
   ht.definedata(ALL: 'YES');
   ht.definedone();

   do until(eof1);
      set DPLocal.&REQUESTID.&RUNID._MasterTable end=eof1;
      format AgeGroup $9.;
      if ht.find()=0 then do;   
         patient = 1;         

         do i=&NUMAGECAT. to 1 by -1;          
            if EpisodeStartDt >= intnx(scan("&AGETYP.",i),birth_date,scan("&AGETHRESH.",i),'sameday') then do;
               AgeGroup = scan("&AGESTRAT.",i,' ');
               leave;
            end;
         end;         
       
         MinAgeDate = intnx(scan("&AGETYP.",1),birth_date,&MINAGE.,'sameday');

         if &MAXAGE.=99999 then MaxAgeDate=intnx('Years',birth_date,110,'sameday');
         else MaxAgeDate=intnx(scan("&AGETYP.",&NUMAGECAT.),birth_date,&MAXAGE.+1,'sameday')-1;       

         Year = year(EpisodeStartDt);
         Month = month(EpisodeStartDt);
         tte=min(eventdt,EpisodeEndDt)-(EpisodeStartDt+BlackoutPer)+1;

         if MinAgeDate <= EpisodeStartDt <= MaxAgeDate then output DPLocal.&REQUESTID.&RUNID._QueryGroup;

      end;

   end; 

   stop;
run;

proc means noprint data = DPLocal.&REQUESTID.&RUNID._QueryGroup noprint;
   var Patient NumDispensing EpisodeRxSup /*MODIF WAVE3*/ EpisodeRxAmt;
   class PatId Group Sex AgeGroup year month;
   id PCHRLSON CHRLSON XCHRLSON NumVisit 
   BMIz_Strat;  /*Identical within episode*/
   output out=_Table00(drop=_freq_ where=(PatId ne "") rename=_type_=seg) 
                  sum(Patient)=Episodes
                  sum(NumDispensing)=Dispensings         
                  sum(EpisodeRxSup)=DaySupp
				  sum(EpisodeRxAmt)=AmtSupp;
run; 

proc means data=_Table00 noprint nway missing;
   var Episodes Dispensings DaySupp /*MODIF WAVE3*/ AmtSupp;
   class Group Sex AgeGroup year month seg PCHRLSON CHRLSON XCHRLSON NumVisit
   BMIz_Strat;
   output out=MSOC.&REQUESTID.&RUNID._NumTab00(drop=_type_ rename=_freq_=Npts where=(Group ne "")) sum=;
run;

/*keeping one record per patient*/
   %MACRO CLASSICTABLES(class=,num=,denomby=,denomwhere=);

/*Member counts in segment*/
proc means data=DPLocal.&REQUESTID.&RUNID._QueryGroup nway noprint;
   var Patient NumDispensing EpisodeRxSup /*MODIF WAVE3*/ EpisodeRxAmt;
   class Patid &class.;
   output out=_Table&num.(drop=_:) sum(Patient)=Episodes           
                                   sum(NumDispensing)=Dispensings         
                                   sum(EpisodeRxSup)=DaySupp
								   sum(EpisodeRxAmt)=AmtSupp;     
run;

proc means data=_Table&num. nway noprint;
   var Episodes Dispensings DaySupp AmtSupp;
   class &class.;      
   output out=Table&num.(drop=_type_ rename=_freq_=Npts) sum=;
run;

/*Add denominators*/
proc means data=_denom nway noprint;
   var inc:;
   class Group &denomby.;
   where &denomwhere.;
   output out=_denom&num.(keep=Group &denomby. Inc:) sum=;
run;

data Table&num.;
   format DPID SITEID $2. &class. NPts Episodes Dispensings DaySupp /*MODIF WAVE3*/ AmtSupp IncDenCount IncDaysCount;
   merge _denom&num. Table&num. ;
   by &class.;
   DpID="&DPID.";                                    
   SiteID="&SITEID.";       
   rename IncDenCount=Denominator IncDaysCount=MemberDays;
   label IncDenCount="Denominator" IncDaysCount="MemberDays";
run;

   %MEND CLASSICTABLES;
   %CLASSICTABLES(class=Group,num=1,denomby=,denomwhere=Segment="ALL");
   %CLASSICTABLES(class=Group AgeGroup,num=2,denomby=AgeGroup,denomwhere=Segment in:("AG"));
   %CLASSICTABLES(class=Group sex,num=3,denomby=sex,denomwhere=Segment="ALL");
   %CLASSICTABLES(class=Group Year,num=4,denomby=year,denomwhere=Segment=:"Y");   
   %CLASSICTABLES(class=Group Year Month,num=5,denomby=year month,denomwhere=Segment in:("PER"));   

/*keeping one record per patient*/
   %MACRO NEWCLASSICTABLES(class=,num=,denomby=,denomwhere=);

/*Member counts in segment*/
proc means data=DPLocal.&REQUESTID.&RUNID._QueryGroup nway noprint missing;
   var Patient NumDispensing EpisodeRxSup /*MODIF WAVE3*/ EpisodeRxAmt;
   class Patid &class.;
   output out=_Table&num.(drop=_:) sum(Patient)=Episodes           
                                   sum(NumDispensing)=Dispensings         
                                   sum(EpisodeRxSup)=DaySupp
								   sum(EpisodeRxAmt)=AmtSupp;     
run;
      
proc means data=_Table&num. nway noprint missing;
   var Episodes Dispensings DaySupp AmtSupp;
   class &class.;      
   output out=Table&num.(drop=_type_ rename=_freq_=Npts) sum=;
run;

   %MEND NEWCLASSICTABLES;

   %LET class=Group NumVisit;
    %LET num=8;

   %NEWCLASSICTABLES(class=Group NumVisit,num=8);   
   %NEWCLASSICTABLES(class=Group PCHRLSON CHRLSON XCHRLSON,num=9);
   %NEWCLASSICTABLES(class=Group PCHRLSON,num=9_1);   
   %NEWCLASSICTABLES(class=Group CHRLSON,num=9_2);   
   %NEWCLASSICTABLES(class=Group XCHRLSON,num=9_3);     
   %NEWCLASSICTABLES(class=Group BMIz_Strat, num=12);
  
proc means noprint data = DPLocal.&REQUESTID.&RUNID._QueryGroup noprint;
   var Patient HadEvent tte;
   class PatId Group Sex AgeGroup year month;
   id PCHRLSON CHRLSON XCHRLSON NumVisit
   BMIz_Strat;  /*Identical within episode*/

   output out=_Table01(drop=_freq_ where=(PatId ne "") rename=_type_=seg) 
                        sum(Patient)=Episodes
                        sum(HadEvent AllEvents)=/*MODIF WAVE3*/ Eps_wEvents All_Events        
                        sum(tte)=tte;
run; 

proc means data=_Table01 noprint nway missing;
   var Episodes /*MODIF WAVE3*/ Eps_wEvents All_Events tte;
   class Group Sex AgeGroup year month seg PCHRLSON CHRLSON XCHRLSON NumVisit
   BMIz_Strat;  /*Identical within episode*/
   output out=MSOC.&REQUESTID.&RUNID._NumTab01(drop=_type_ rename=_freq_=Npts where=(Group ne "")) sum=;
run;

   %MACRO NEWTABLES(class=,num=,denomby=,denomwhere=);

proc means data=DPLocal.&REQUESTID.&RUNID._QueryGroup nway noprint;
   var Patient HadEvent tte;
   class Patid &class.;
   output out=_Table&num.(drop=_:) sum(Patient)=Episodes           
                                   sum(HadEvent AllEvents)=/*MODIF WAVE3*/ Eps_wEvents All_Events          
                                   sum(tte)=tte;     
run;


proc means data=_Table&num. nway noprint;
   var Episodes /*MODIF WAVE3*/ Eps_wEvents All_Events tte;
   class &class.;      
   output out=Table&num.(drop=_type_ rename=_freq_=Npts) sum(Episodes)=Episodes
                                                         sum(/*MODIF WAVE3*/ Eps_wEvents All_Events )=Eps_wEvents All_Events 
                                                         sum(tte)=DaysAtRisk;
run;

/*Add denominators*/
proc means data=_denom nway noprint;
   var inc:;
   class Group &denomby.;
   where &denomwhere.;
   output out=_denom&num.(keep=Group &denomby. Inc:) sum=;
run;

data Table&num.;
   format DPID SITEID $2. &class. NPts Episodes /*MODIF WAVE3*/ Eps_wEvents All_Events DaysAtRisk IncDenCount IncDaysCount;
   merge _denom&num. Table&num. ;
   by &class.;
   DpID="&DPID.";                                    
   SiteID="&SITEID.";       
   rename IncDenCount=Denominator IncDaysCount=MemberDays;
   label IncDenCount="Denominator" IncDaysCount="MemberDays";
run;

   %MEND NEWTABLES;
   %NEWTABLES(class=Group,num=6,denomby=,denomwhere=Segment="ALL");
   %NEWTABLES(class=Group Year,num=7,denomby=year,denomwhere=Segment=:"Y");

   %MACRO NewNEWTABLES(class=,num=,denomby=,denomwhere=);

proc means data=DPLocal.&REQUESTID.&RUNID._QueryGroup nway noprint missing;
   var Patient HadEvent AllEvents tte;
   class Patid &class.;
   output out=_Table&num.(drop=_:) sum(Patient)=Episodes           
                                   sum(HadEvent AllEvents)=/*MODIF WAVE3*/ Eps_wEvents All_Events          
                                   sum(tte)=tte;     
run;


proc means data=_Table&num. nway noprint missing;
   var Episodes /*MODIF WAVE3*/ Eps_wEvents All_Events  tte;
   class &class.;      
   output out=Table&num.(drop=_type_ rename=_freq_=Npts) sum(Episodes)=Episodes
                                                         sum(Eps_wEvents All_Events )=/*MODIF WAVE3*/ Eps_wEvents All_Events 
                                                         sum(tte)=DaysAtRisk;
run;

   %MEND NewNEWTABLES;

   %LET class=Group NumVisit;
   %LET num=10;

   %NewNEWTABLES(class=Group NumVisit,num=10);   
   %NewNEWTABLES(class=Group PCHRLSON CHRLSON XCHRLSON,num=11);   
   %NewNEWTABLES(class=Group PCHRLSON,num=11_1);   
   %NewNEWTABLES(class=Group CHRLSON,num=11_2);   
   %NewNEWTABLES(class=Group XCHRLSON,num=11_3);   
   %NewNEWTABLES(class=Group BMIz_Strat, num=12);

   /*MODIF WAVE3 Block Start*/
   * Table 13;
   proc means noprint data = DPLocal.&REQUESTID.&RUNID._QueryGroup nway noprint;
   var Patient AllEvents;
   class PatId Group;  
   id Patient;
   output out=Table13(drop=_: where=(PatId ne "")) 
                  	  sum(Patient)=NumEpisodes
                  	  sum(AllEvents)=TotEvents;
   run; 

   proc means noprint data = Table13 nway noprint;
   var Patient;
   class Group NumEpisodes TotEvents;     
   output out=Table13(drop=_:) 
                  	  sum(Patient)=TotMembers;                  
   run; 

   proc means noprint data = Table13 nway noprint;   
   var TotMembers;
   class Group;
   output out=_totalnum(drop=_:) Sum=IncNumCount;                  
   run;

   proc means data=_denom nway noprint;
   var IncDenCount;
   class Group;
   where segment="ALL";
   output out=_totalden(keep=Group IncDenCount) sum=;
   run;

   data _numden;
   retain Group NumEpisodes TotEvents TotMembers;
   merge _totalnum _totalden;
   by Group;
   NumEpisodes=0;
   TotEvents=0;
   TotMembers=IncDenCount-IncNumCount;
   drop IncDenCount IncNumCount;
   run;

   data Table13;
   set _numden
   	   Table13;
   run;

   * Table 14;
   proc means noprint data = DPLocal.&REQUESTID.&RUNID._QueryGroup nway noprint;
   var Patient;
   class Group PatId AllEvents;  
   id Patient; 
   output out=Table14(drop=_: where=(PatId ne "") rename=AllEvents=EventsPerEp) sum(Patient)=NumEpisodes;
   run; 

   proc means noprint data = Table14 nway noprint;
   var NumEpisodes Patient;
   class Group EventsPerEp;     
   output out=Table14(drop=_:) sum(NumEpisodes Patient)=NumEpisodes TotMembers;                  
   run; 

   /*MODIF WAVE3 Block End*/

/*---------------------------------------------------------------------------*/
/*  15 -- Export tables and data                                             */
/*---------------------------------------------------------------------------*/

   %MACRO export(DATAPARTNER,REQUESTID,NAME,VARTO); /*To export in .csv and .sas7bdat format*/

      %IF %INDEX(&name.,Table2) %THEN %DO;

data &NAME.;
   set &NAME.;
   AgeGroup=TRANWRD(AgeGroup,'-',' to ');
run;

      %END;

/*exporting raw tables into one .lst file*/
options nodate nonumber formdlim="-";

/*Select Table Name*/
data _Title;
   set titles;
   if table="&NAME." then do;
      &VARTO.=title;
      call symput('TITLE1',title);output;
      &VARTO.="Source: &REQUESTID.&RUNID.";
      call symput('TITLE2',&VARTO.);
      output;
   end;
   keep &VARTO.;
run;

TITLE1 "&TITLE1.";
TITLE2 "&TITLE2.";

/*Print results in .lst*/
proc print data=&name. noobs;run;

data MSOC.&REQUESTID.&RUNID._&name.;
      set  &NAME.; 
run;

data &NAME.;
      format &varto. $100.;
      set &NAME. _Title;
run;

PROC EXPORT DATA= &NAME. 
            OUTFILE= "&DPLOCAL.&REQUESTID.&RUNID._&NAME..csv" 
                   DBMS=CSV  REPLACE;
RUN;

   %MEND export;

   %MACRO WRAPPER;

proc printto print="&MSOC.&REQUESTID.&RUNID._all_tables.lst" new;
run;

/*Not tabulated to accommodate partners who cannot read more than 120 long lines*/
Data Titles;
   Format table $9. title $100.;

   Table="Table1";
   Title="Table 1: Counts of Members, Episodes, Dispensing and Days of Supply - &STARTDATE.";
   output;

   Table="Table2";
   Title="Table 2: Counts of Members, Episodes, Dispensing and Days of Supply Per Age Group - &STARTDATE.";
   output;

   Table="Table3";
   Title="Table 3: Counts of Members, Episodes, Dispensing and Days of Supply by Sex - &STARTDATE.";
   output;

   Table="Table4";
   Title="Table 4: Counts of Members, Episodes, Dispensing and Days of Supply by Year - &STARTDATE.";
   output;

   Table="Table5";
   Title="Table 5: Counts of Members, Episodes, Dispensing and Days of Supply by Year and Month - &STARTDATE.";
   output;

   Table="Table6";
   Title="Table 6: Frequency of Events/Outcomes among Incident Drug/Procedure Users - &STARTDATE.";
   output;

   Table="Table7";
   Title="Table 7: Frequency of Events/Outcomes among Incident Drug/Procedure Users by Year - &STARTDATE.";
   output;

   Table="Table8";
   Title="Table 8: Counts of Members, Episodes, Dispensing and Days of Supply by Number of Visits - &STARTDATE.";
   output;

   Table="Table9";
   Title="Table 9: Counts of Members, Episodes, Dispensing and Days of Supply by Charlson Comorbidity Index - &STARTDATE.";
   output;

   Table="Table9_1";
   Title="Table 9.1: Counts of Members, Episodes, Dispensing and Days of Supply by Charlson Comorbidity Index (PCHRLSON) - &STARTDATE.";
   output;

   Table="Table9_2";
   Title="Table 9.2.: Counts of Members, Episodes, Dispensing and Days of Supply by Charlson Comorbidity Index (CHRLSON) - &STARTDATE.";
   output;

   Table="Table9_3";
   Title="Table 9.3.: Counts of Members, Episodes, Dispensing and Days of Supply by Charlson Comorbidity Index (XCHRLSON) - &STARTDATE.";
   output;

   Table="Table10";
   Title="Table 10: Frequency of Events/Outcomes among Incident Drug/Procedure Users by Number of Visits- &STARTDATE.";
   output;

   Table="Table11";
   Title="Table 11: Frequency of Events/Outcomes among Incident Drug/Procedure Users by Charlson Comorbidity Index - &STARTDATE.";
   output;

   Table="Table11_1";
   Title="Table 11.1: Frequency of Events/Outcomes among Incident Drug/Procedure Users by Charlson Comorbidity Index (PCHRLSON) - &STARTDATE.";
   output;

   Table="Table11_2";
   Title="Table 11.2: Frequency of Events/Outcomes among Incident Drug/Procedure Users by Charlson Comorbidity Index (CHRLSON) - &STARTDATE.";
   output;

   Table="Table11_3";
   Title="Table 11.3: Frequency of Events/Outcomes among Incident Drug/Procedure Users by Charlson Comorbidity Index (XCHRLSON) - &STARTDATE.";
   output;

   Table="Table12";
   Title="Table 12: Frequency of Events/Outcomes among Incident Drug/Procedure Users by BMI Z Score Change - &STARTDATE.";
   output;
/*MODIF WAVE3*/
   Table="Table13";
   Title="Table 13: Number of members by the number of valid treatment episodes and total number of events - &STARTDATE.";
   output;

   Table="Table14";
   Title="Table 14: Number of valid treatment episodes by the number of events per episode and total number of members - &STARTDATE.";
   output;
    
   Table="signature";
   Title="Signature of Request - &STARTDATE.";
   output;
   
run;   
     
      %DO i=1 %TO 14;
         %EXPORT(&DPID.&SITEID.,&REQUESTID.,Table&i.,Group);
      %END; 
      %EXPORT(&DPID.&SITEID.,&REQUESTID.,Table9_1,Group);
      %EXPORT(&DPID.&SITEID.,&REQUESTID.,Table9_2,Group);
      %EXPORT(&DPID.&SITEID.,&REQUESTID.,Table9_3,Group);
      %EXPORT(&DPID.&SITEID.,&REQUESTID.,Table11_1,Group);
      %EXPORT(&DPID.&SITEID.,&REQUESTID.,Table11_2,Group);
      %EXPORT(&DPID.&SITEID.,&REQUESTID.,Table11_3,Group);
   %MEND WRAPPER;
   %WRAPPER;

data _NULL_;
   temp=DATETIME();
   seconds=temp-&start.;
   hours=int(seconds/3600);
   minutes=int((seconds-hours*3600)/60);
   seconds2=int((seconds-hours*3600-minutes*60));
   call symput('STOP',temp);
   call symput('totalseconds',put(seconds,best.));
   call symput('hours',put(hours,4.0));
   call symput('minutes',put(minutes,2.0));
   call symput('seconds',put(seconds2,2.0));
run;

%PUT TOTAL RUN TIME was &hours. h &minutes. m &seconds. s; 


/*********************************/
/* Create Request Signature File */
/*********************************/

data signature;
   MPNum="&MPNum.";
   MPVer="&MPVer.";
   DPID="&DPID.";                                    
   SITEID="&SITEID.";                                
   RequestID="&REQUESTID.";    
   RunID="&RUNID.";
   format StartTime StopTime datetime21.2; 
   StartTime=trim(left(&START.)); 
   StopTime=trim(left(&STOP.));   
   format RunTime Seconds $20.;
   RunTime="&hours. h &minutes. m &seconds. s";
   Seconds="&totalseconds. s";
   ScenarioCnt=strip("&NQUERYGROUPS.");
   ENROLGAP="&ENROLGAP.";   
   COVERAGE="&COVERAGE.";
   QUERYFROM="&QUERYFROMc.";                            
   QUERYTO="&QUERYTOc.";
   QUERYFILE="&QUERYFILE.";
   INCQUERYFILE="&INCQUERYFILE.";
   QUERYEVENTFILE="&QUERYEVENTFILE.";
   INCEVENTFILE="&INCEVENTFILE.";
   CONDFILE="&CONDFILE.";
   AGESTRAT="&AGESTRAT.";
   CCIFILE="&CCIFILE.";
   UTILFILE="&UTILFILE.";
   BMIKIDSFILE="&BMIKIDSFILE.";
   OUTTABLESFILE="&OUTTABLESFILE.";
   /*MODIF WAVE3*/
   STOCKPILINGFILE="&STOCKPILINGFILE.";
   output;
run;

proc transpose data=signature out=signature(rename=_NAME_=Var rename=COL1=VALUE);
   var _ALL_;
run;

%EXPORT(&DPID.&SITEID.,&RequestID.,signature,var);

%MACRO REMOVETABLE(table=);
	%if %sysfunc(fileexist(&msoc.&REQUESTID.&RUNID._&table..sas7bdat))=1 %then %do;
		proc datasets library=msoc nolist nowarn;
   			delete &REQUESTID.&RUNID._&table.;
    	quit;
	%end;
%MEND REMOVETABLE;

%MACRO OUTPUTSELECTION;	
	%IF %UPCASE("&OUTTABLESFILE.") ne %STR("") %THEN %DO;
		data _null_;
	     	set _OUTTABLESFILE;

	     	strng=cats('%REMOVETABLE(table=', TabName,');');

	     	if substr(upcase(TabRequired),1,1) ne "Y" then call execute(%nrstr(strng));
		run;
	%END;
%MEND OUTPUTSELECTION;
%OUTPUTSELECTION;
  
proc datasets library=work nolist nowarn;
   delete _:;
quit;
   
proc printto log=log print=print;
run;

%PUT TOTAL RUN TIME was &hours. h &minutes. m &seconds. s; 

%MEND MODULARPROGRAM3;

/*---------------------------------------------------------------------------*/
/*  16 --  Example of invoking Modular Program Macro                         */
/*---------------------------------------------------------------------------*/

	  %MODULARPROGRAM3
      (
            REQUESTID=mp3,
			RUNID=r01,
            ENROLGAP=45,
			COVERAGE=MD,
            QUERYFROM=01/01/2008,
            QUERYTO=12/31/2010,
            QUERYFILE=mp3_wave3_query.sas7bdat,
            INCQUERYFILE=,
            QUERYEVENTFILE=mp3_wave3_event.sas7bdat,
            INCEVENTFILE=,
            CONDFILE=,
            AGESTRAT=35-49 50-64 65-79 80+ ,
            CCIFILE=,
            UTILFILE=,            
            BMIKIDSFILE=,
			OUTTABLESFILE=mp3_output_tables.sas7bdat,
			STOCKPILINGFILE=mp3_stockpiling.sas7bdat
      );
